#include <memory.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <cstdarg>
#include <string>
#include <vector>

// Enthält alle Funktionen der Datafox API zur Gerätekonfiguration.
#include "../include/api/df_api.h"
#include "../include/api/df_setup_records.h"

#include "../../shared/include/common/error_defines.h"

bool bVerbose = false;
bool bPrintListen = false;
bool bReadRecords = false;
bool bReadSingleRecord = false;
bool bReadLog = false;

FILE * pOutputFile = nullptr;


void my_printf( const char* fmt... )
{
    va_list args;
    va_start(args, fmt);

    if ( pOutputFile )
    {
        vfprintf( pOutputFile, fmt, args );
    }
    vprintf( fmt, args );
    va_end( args );
}

const char * getArgument( int p_nIdx, int p_nArgC, char const * const * const p_ppchArgV )
{
    return ( p_nIdx < p_nArgC ? p_ppchArgV[ p_nIdx ] : nullptr );
}

void printHexMem( const char * pBuffer, int nSize )
{
    for ( int nOffs = 0; nOffs < nSize; nOffs += 16 )
    {
        my_printf( "%6i: ", nOffs );
        for ( int nPos = 0; nPos < 16; ++nPos )
        {
            if ( nOffs + nPos < nSize )
            {
                my_printf( " %02X", pBuffer[ nOffs + nPos ] );
            }
            else
            {
                my_printf( "   " );
            }
        }

        my_printf( " " );

        for ( int nPos = 0; nPos < 16; ++nPos )
        {
            if ( nOffs + nPos < nSize )
            {
                if ( pBuffer[ nOffs + nPos ] >= 32 )
                {
                    my_printf( "%c", pBuffer[ nOffs + nPos ] );
                }
                else
                {
                    my_printf( "_" );
                }
            }

        }
        my_printf( "\n" );
    }
}

void printDfString( const char * const p_pBuffer )
{
    bool bLastCharWasCRLF = false;
    const char * pBuffer = p_pBuffer;
    while ( *pBuffer )
    {
        bool bIsCrLf = ( *pBuffer == '\n' ) || ( *pBuffer == '\r' );
        if ( ( bLastCharWasCRLF != bIsCrLf ) && bIsCrLf )
        {
            my_printf( "\n" );
        }
        else if ( !bIsCrLf )
        {
            my_printf( "%c", *pBuffer );

        }
        bLastCharWasCRLF = bIsCrLf;
        ++pBuffer;
    }

    if ( bVerbose )
    {
        my_printf( "\n\nHEX:\n" );
        printHexMem( p_pBuffer, strlen( p_pBuffer ) );
        my_printf( "\n.\n\n" );
    }
}

const char * listTypeId2Description( int p_nTypeId )
{
    switch ( p_nTypeId )
    {
    case 1:  // uint32 (4 Byte)
        return "uint(%i)";
    case 2:  // DateTime (7 Byte, 4 Byte Date, 3 Byte Time)
        return "datetime(%i)";
    case 3:  // String (1-41 Byte, only digits [0-9])
        return "num-string(%i)";
    case 4:  // String (1-41 Byte, Latin1 codec)
        return "latin1-string(%i)";
    case 5:  // Date (4 Byte; Y100, Y, M, D)
        return "date(%i)";
    case 6:  // Time (3 Byte; h, m, s)
        return "time(%i)";
    case 7:  // Finger template (161 Byte; DIN V66400)
        return "finger(%i)";
    case 8:  // Finger template (216 Byte; Idencom Compact)
        return "finger(%i)";
    case 9:  // Binar data (1-220 Byte, Binär)
        return "binary(%i)";
    case 10: // Daten (2-241 Byte, Binär) :
        return "data(%i)";
        break;
    }

    return "?";

}

enum results_t
{
    ERR_NO_ERROR = 0,
    ERR_NOT_IMPLEMENTED,
    ERR_INSUFFICIENT_PARAMETER,
    ERR_INVALID_ARGUMENT,
    ERR_FILE_CREATE_ERROR,
    ERR_DFCOM_LOAD_FAILED,
    ERR_DFCOM_OPEN_FAILED,
    ERR_DFCOM_SEND_MESSAGE,
    ERR_DFCOM_UNZIP_DFC_FAILED,
    ERR_DFCOM_DFCUPLOAD_FAILED,
    ERR_DFCOM_DFCSETUPLADEN_FAILED,
    ERR_DFCOM_LIST_LOADLISTDESCRIPTION_ERROR,
    ERR_DFCOM_LIST_UPLOAD_FAILED,
    ERR_DFCOM_LIST_FILE_NOT_FOUND,
    ERR_DFCOM_LIST_ACCES_DESCRIPTION_ERROR,
    ERR_DFCOM_LIST_MALLOC_ERROR,
    ERR_DFCOM_LIST_LINE_TOO_LONG,
    ERR_DFCOM_LIST_COLUMN_TYPE_UNSUPPORTED,
    ERR_DFCOM_LIST_COLUMN_WIDTH_EXCEEDED,
    ERR_DFCOM_LIST_MAKELISTE_FAILED,
    ERR_DFCOM_DFCLISTLOAD_FAILED,
    ERR_DFCOM_DFCLOADRECORDDESCRIPTION_FAILED,
    ERR_DFCOM_DFCQUITRECORD_FAILED,
    ERR_DFCOM_DFCREADRECORD_FAILED,
    ERR_DFCOM_DFCREADSYSVAR_FAILED,
    ERR_DFCOM_DFCWRITESYSVAR_FAILED,
    ERR_DFCOM_DFCREADSYSRECORD_FAILED,
    ERR_DFCOM_DFCQUITSYSRECORD_FAILED,
    ERR_DFCOM_DFCTABLE_OPEN,
    ERR_DFCOM_DFCTABLE_GETCOUNTS

};

struct backlightParameter_t
{
    int nId = -1;
    int nDur = 0;
    int nRR = 0;
    int nGG = 0;
    int nBB = 0;
    int nWW = 0;
};

struct firmwareUploadParameter_t
{
    std::string strFirmwareFilename;
};

struct setupUploadParameter_t
{
    std::string strSetupFilename;
};

struct listParameter_t
{
    int         nListMode = 0; // 1 == Setup-Liste, 2 == Zutrittskontroll-Liste
    std::string strListName;
    std::string strListFilename;
};

int prepareList( int connection_id, int device_address, const listParameter_t & lp )
{
    int nRes = ERR_NO_ERROR;

    int arrFieldSizes[ 25 ];
    int arrFieldTypes[ 25 ];
    memset( arrFieldSizes, 0, 25 * sizeof( int ) );
    memset( arrFieldTypes, 0, 25 * sizeof( int ) );
    int nListRowSize = 0;

    const bool bIsAc2List = lp.nListMode == 2;

    int nErrCode = 0;
    if ( !DFCLoadListenbeschreibung( connection_id, device_address,  &nErrCode ) )
    {
        my_printf( "ERROR %i while loading the device's list description\n", nErrCode );
        nRes = ERR_DFCOM_LIST_LOADLISTDESCRIPTION_ERROR;
    }

    int nListIndexMatch = -1;
    int nFieldCnt = 0;
    if ( nRes == ERR_NO_ERROR )
    {
        // Zutrittslisten laufen von 200..207
        int nListIdx = ( bIsAc2List ? 200 : 0 );
        const int nListMaxIndex = ( bIsAc2List ? 208 : DFCListBCnt( connection_id ) );

        if ( bVerbose )
        {
            my_printf( "Looking for list name '%s'\n", lp.strListName.c_str() );
        }

        for ( ; nListIdx < nListMaxIndex; ++nListIdx )
        {
            char arrName[ 20 ];
            memset( arrName, 0, sizeof( arrName ) );
            DFCListBDatensatz( connection_id, nListIdx, arrName, &nFieldCnt, 0 );

            // Listennamen prüfen
            bool bMatch = strncmp( lp.strListName.c_str(), arrName, 17 ) == 0;

            if ( bMatch )
            {
                nListIndexMatch = nListIdx;
                if ( bVerbose )
                {
                    my_printf( "*** Match at list index %i\n", nListIndexMatch );
                    my_printf( "*** List name specified: '%s', found '%s'\n", lp.strListName.c_str(), arrName );
                }

                for ( int nCol = 0; nCol < nFieldCnt; ++nCol )
                {
                    char arrColName[ 20 ];
                    memset( arrColName, 0, sizeof( arrName ) );

                    int nRes = DFCListBFeld( connection_id, nListIdx, nCol, arrColName, arrFieldTypes + nCol, arrFieldSizes + nCol );
                    if ( !nRes )
                    {
                        my_printf( "#### FEHLER beim Zugriff auf Spalte %i\n", nCol );
                        nRes = ERR_DFCOM_LIST_ACCES_DESCRIPTION_ERROR;
                    }
                    else
                    {
                        if ( bVerbose )
                        {
                            my_printf( "**** COL %i: '%s', TYPE %i, SIZE %i\n", nCol, arrColName, arrFieldTypes[ nCol ], arrFieldSizes[ nCol ] );
                        }

                        nListRowSize += arrFieldSizes[ nCol ];
                    }
                }

                if ( bVerbose )
                {
                    my_printf( "Total row width %i\n", nListRowSize );
                }
                break;
            }
        }
    }

    // Anzahl der Zeilen in der Datei zählen
    int nRowsInFileCount = 0;
    FILE * pFile = 0;
    if ( nRes == ERR_NO_ERROR )
    {
        if ( nListIndexMatch >= 0 )
        {
            pFile = fopen( lp.strListFilename.c_str(), "r" );
            if ( pFile )
            {
                bool bIsCrLf = false;
                while ( !feof( pFile ) )
                {
                    const int nCh = fgetc( pFile );
                    if ( nCh == EOF )
                        break;

                    const bool bCurrentCrLf = ( nCh == 10 || nCh == 13 );
                    if ( bIsCrLf != bCurrentCrLf )
                    {
                        if ( bCurrentCrLf )
                        {
                            ++nRowsInFileCount;
                        }
                        bIsCrLf = bCurrentCrLf;
                    }
                }
                if ( !bIsCrLf )
                {
                    // Letzte Zeile nicht mit CR/LF abgeschlossen? Trotzdem mitzählen
                    ++nRowsInFileCount;
                }
            }
            else
            {
                my_printf( "ERROR List data file '%s' not found\n", lp.strListFilename.c_str() );
                nRes = ERR_DFCOM_LIST_FILE_NOT_FOUND;
            }

        }
        else
        {
            my_printf( "ERROR List '%s' not present on device\n", lp.strListName.c_str() );
            nRes = ERR_DFCOM_LIST_UPLOAD_FAILED;
        }

    }

    if ( bVerbose )
    {
        my_printf( "**** File contains %i rows of data\n", nRowsInFileCount );
    }

    char * pBuffer = 0;

    if ( nRes == ERR_NO_ERROR )
    {
        pBuffer = ( char * ) malloc( nListRowSize * nRowsInFileCount );
        if ( pBuffer )
        {
            memset( pBuffer, 0, nListRowSize * nRowsInFileCount );
        }
        else
        {
            my_printf( "ERROR Memory allocation error - could not allocate %i bytes\n", nListRowSize * nRowsInFileCount  );
            nRes = ERR_DFCOM_LIST_MALLOC_ERROR;
        }
    }

    if ( nRes == ERR_NO_ERROR )
    {
        fseek( pFile, 0, SEEK_SET );

        char buffer[ 256 + 1 ];
        int  nIdx = 0;
        memset( buffer, 0, sizeof ( buffer ) );

        bool bIsCrLf = false;
        int nLine = 0;
        while ( !feof( pFile ) )
        {
            const int nCh = fgetc( pFile );
            const bool bCurrentCrLf = ( nCh == 10 || nCh == 13 );
            if ( bIsCrLf != bCurrentCrLf )
            {
                if ( bCurrentCrLf )
                {
                    // process line buffer
                    if ( bVerbose )
                    {
                        my_printf( "READ '%s'\n", buffer );
                    }

                    int nRowOffset = 0;
                    const int nBaseOffset = nListRowSize * nLine;
                    char * pBufferPtr = buffer;
                    for ( int nCol = 0; nCol < nFieldCnt; ++nCol )
                    {
                        char * pFieldEndPtr = strchr( pBufferPtr, '\t' );

                        int nSize = 0;
                        if ( pFieldEndPtr )
                        {
                            nSize = pFieldEndPtr - pBufferPtr;
                        }
                        else
                        {
                            nSize = strlen( pBufferPtr );
                            pFieldEndPtr = pBufferPtr + nSize;
                        }
                        *pFieldEndPtr = 0;
                        if ( bVerbose )
                        {
                            my_printf( "Field %i: %i byte '%s'\n", nCol, nSize, pBufferPtr );
                        }

                        // Feldbreite prüfen
                        const int nFieldSize = arrFieldSizes[ nCol ];
                        if ( nSize >= nFieldSize )
                        {
                            my_printf( "ERROR Line %i, Field %i ('%s'): field size exceeded: max size (incl. termination) is %i, data bytes supplied %i\n", nLine, nCol, pBufferPtr, nFieldSize, nSize );
                            nRes = ERR_DFCOM_LIST_COLUMN_WIDTH_EXCEEDED;
                            break;
                        }

                        switch ( arrFieldTypes[ nCol ] )
                        {
                        case 3:  // String (1-41 Byte, only digits [0-9])
                        case 4:  // String (1-41 Byte, Latin1 codec)
                            strncpy( pBuffer + nBaseOffset + nRowOffset, pBufferPtr, nSize );
                            break;

                        case 1:  // uint32 (4 Byte)
                        case 2:  // DateTime (7 Byte, 4 Byte Date, 3 Byte Time)
                        case 5:  // Date (4 Byte; Y100, Y, M, D)
                        case 6:  // Time (3 Byte; h, m, s)
                        case 7:  // Finger template (161 Byte; DIN V66400)
                        case 8:  // Finger template (216 Byte; Idencom Compact)
                        case 9:  // Binar data (1-220 Byte, Binär)
                        case 10: // Daten (2-241 Byte, Binär) :
                            my_printf( "ERROR List has unsupported column type %i for column %i\n", arrFieldTypes[ nCol ], nCol );
                            nRes = ERR_DFCOM_LIST_COLUMN_TYPE_UNSUPPORTED;
                            break;
                        }
                        if ( bVerbose )
                        {
                            my_printf( "Data stored in record field: '%s'\n", pBuffer + nBaseOffset + nRowOffset );
                        }

                        if ( nRes != ERR_NO_ERROR )
                            break;

                        pBufferPtr = pFieldEndPtr + 1;
                        nRowOffset += nFieldSize;
                    }

                    if ( bVerbose )
                    {
                        my_printf( "FINAL LINE BUFFER:" );
                        printHexMem( pBuffer + nBaseOffset, nListRowSize );
                        my_printf( "\n" );
                    }

                    ++nLine;
                    nIdx = 0;
                    memset( buffer, 0, sizeof ( buffer ) );
                }
                bIsCrLf = bCurrentCrLf;
            }

            if ( !bCurrentCrLf )
            {
                if ( nIdx < 256 )
                {
                    buffer[ nIdx ] = nCh;
                }
                else if ( nIdx == 256 ) // Nur eine Warnung für diese Zeile...
                {
                    my_printf( "ERROR Line %i exeeds maximum length\n", nLine );
                    nRes = ERR_DFCOM_LIST_LINE_TOO_LONG;
                }
                nIdx++;
            }

        }
    }

    if ( nRes == ERR_NO_ERROR )
    {
        nErrCode = 0;
        int nMakeListRes = ( bIsAc2List ?
                                 DFCMakeEntrance2List( connection_id, nListIndexMatch - 200, nRowsInFileCount, nListRowSize * nRowsInFileCount, (unsigned char *) pBuffer, &nErrCode ) :
                                 DFCMakeListe( connection_id, nListIndexMatch, nRowsInFileCount, nListRowSize * nRowsInFileCount, (unsigned char *) pBuffer, 0 ) );
        if ( nMakeListRes == 0 )
        {
            my_printf( "ERROR during DFCMakeListe()/DFCMakeEntrance2List() for list '%s'\n", lp.strListName.c_str() );
            nRes = ERR_DFCOM_LIST_MAKELISTE_FAILED;
        }
    }

    return nRes;
}

void printListe( int connection_id, int listen_id )
{
    char arrName[ 20 ];
    memset( arrName, 0, sizeof( arrName ) );
    int nFieldCnt = -1;
    DFCListBDatensatz( connection_id, listen_id, arrName, &nFieldCnt, 0 );

    my_printf( "List index %i: '%s', number of columns = %i\n", listen_id, arrName, nFieldCnt );

    int nListRowSize = 0;
    for ( int nCol = 0; nCol < nFieldCnt; ++nCol )
    {
        int nFieldType = -1;
        int nFieldSize = -1;
        char arrColName[ 20 ];
        memset( arrColName, 0, sizeof( arrName ) );

        int nRes = DFCListBFeld( connection_id, listen_id, nCol, arrColName, &nFieldType, &nFieldSize );
        if ( !nRes )
        {
            my_printf( "#### Error accessing column index %i\n", nCol );
        }
        else
        {
            my_printf( "    Column %i: '%s', Type ", nCol, arrColName );
            my_printf( listTypeId2Description( nFieldType ), nFieldSize );
            my_printf( "\n" );
            nListRowSize += nFieldSize;
        }
    }

    my_printf( "    Total row width %i\n\n", nListRowSize );
}

int main( int argc, char *argv[] )
{
    int nRes = ERR_NO_ERROR;

    // Kommandozeilenargumente analysieren
    bool bDump = false;                 // --dump oder -d gesetzt?
    bool bHelpRequested = false;        // --help oder -h gesetzt?
    int nInvalidArgIndex = 0;           // Position des ersten, unbekannten Parameters
    int connection_id                  = 1;                      // Jede Verbindung benötigt ihre eindeutige Verbindungsnummer.
                                                                 // Die Verbindungs-Id ist bei jedem Kommunikationsbefehl als erstes
                                                                 // Funktionsargument anzugeben.
    int device_address                 = 254;                    // Gerätenummer für mehrere Geräte in einem Bus wie RS485 Verbindungen.
                                                                 // Bei Direktverbindung mit dem Gerät wie bei integriertem USB oder TCP/IP
                                                                 // ist diese Gerätenummer immer fest mit 254 anzugeben.
                                                                 // Die Gerätenummer ist bei jedem Befehl der mit dem Gerät kommuniziert als zweites
                                                                 // Funktionsargument anzugeben.
    int connection_type                = 1;                      // Gewählte Verbindungsart. 1 = Seriell (USB), 3 = TCP/IP
    char connection_string[64]         = { 'C', 'O', 'M', '3', 0 }; // Je nach Verbindungsart, 1 = Serielle Schnittstelle, 3 = IP oder Hostname.
    int connection_value               = 38400;                  // Je nach Verbindungsart, 1 = Baudrate, 3 = Portnummer.
    int communication_read_timeout     = 800;

    std::string                                             strOutputFilename;
    std::vector< backlightParameter_t >                     backlight;
    firmwareUploadParameter_t                               firmwareUpload;
    setupUploadParameter_t                                  setupUpload;
    std::vector< listParameter_t >                          listParameter;
    std::vector< listParameter_t >                          listenLesenParameter;
    std::vector< std::string>                               vecReadSystemVariables;
    std::vector< std::pair< std::string, std::string > >    vecWriteSystemVariables;

    if ( nRes == ERR_NO_ERROR )
    {
        for ( int nIdx = 1; nIdx < argc; ++nIdx )
        {
            // --dump
            if ( !strcmp( "-d", argv[ nIdx ] ) || !strcmp( "--dump", argv[ nIdx ] ) )
            {
                bDump = true;
            }

            // --print-lists
            else if ( !strcmp( "--print-lists", argv[ nIdx ] ) )
            {
                bPrintListen = true;
            }
            // --read-device-log
            else if ( !strcmp( "--read-device-log", argv[ nIdx ] ) )
            {
                bReadLog = true;
            }

            // --read-record
            else if ( !strcmp( "--read-record", argv[ nIdx ] ) )
            {
                bReadSingleRecord = true;
            }

            // --read-records
            else if ( !strcmp( "--read-records", argv[ nIdx ] ) )
            {
                bReadRecords = true;
            }

            // --help
            else if ( !strcmp( "-h", argv[ nIdx ] ) || !strcmp( "--help", argv[ nIdx ] ) )
            {
                bHelpRequested = true;
            }

            // --verbose -v
            else if ( !strcmp( "-v", argv[ nIdx ] ) || !strcmp( "--verbose", argv[ nIdx ] ) )
            {
                bVerbose = true;
            }

            // --output <filename>
            else if ( !strcmp( "-o", argv[ nIdx ] ) || !strcmp( "--output", argv[ nIdx ] ) )
            {
                const char * pchFilename = getArgument( nIdx + 1, argc, argv );
                if ( pchFilename )
                {
                    strOutputFilename = pchFilename;
                    nIdx += 1;
                }
                else
                {
                    nRes = ERR_INSUFFICIENT_PARAMETER;
                }
            }

            // --serial-device <com-port>
            else if ( !strcmp( "-s", argv[ nIdx ] ) || !strcmp( "--serial-device", argv[ nIdx ] ) )
            {
                connection_type = 1;
                connection_value = 38400;
                communication_read_timeout = 800;
                const char * pchDevice = getArgument( nIdx + 1, argc, argv );

                if ( pchDevice )
                {
                    strncpy( connection_string, pchDevice, 63 );
                    nIdx += 1;
                }
                else
                {
                    nRes = ERR_INSUFFICIENT_PARAMETER;
                }
            }

            // --tcp-device <address> <port>
            else if ( !strcmp( "-t", argv[ nIdx ] ) || !strcmp( "--tcp-device", argv[ nIdx ] ) )
            {
                connection_type = 3;
                connection_value = 8000;
                communication_read_timeout = 3000;
                const char * pchDevice = getArgument( nIdx + 1, argc, argv );
                const char * pchPort   = getArgument( nIdx + 2, argc, argv );

                if ( pchDevice && pchPort )
                {
                    strncpy( connection_string, pchDevice, 63 );
                    connection_value = atoi( pchPort );
                    nIdx += 2;
                }
                else
                {
                    nRes = ERR_INSUFFICIENT_PARAMETER;
                }
            }

            // --communication-timeout <time ms>
            else if ( !strcmp( "--communication-timeout", argv[ nIdx ] ) )
            {
                const char * pchTimeout = getArgument( nIdx + 1, argc, argv );
                if ( pchTimeout )
                {
                    communication_read_timeout = atoi( pchTimeout );
                    nIdx += 1;
                }
                else
                {
                    nRes = ERR_INSUFFICIENT_PARAMETER;
                }
            }

            // --baud-rate <baudrate>
            else if ( !strcmp( "--baud-rate", argv[ nIdx ] ) )
            {
                const char * pchBaudrate = getArgument( nIdx + 1, argc, argv );
                if ( pchBaudrate )
                {
                    connection_value = atoi( pchBaudrate );
                    nIdx += 1;
                }
                else
                {
                    nRes = ERR_INSUFFICIENT_PARAMETER;
                }
            }

            // --backlight <id> <duration> <red> <green> <blue> <white>
            else if ( !strcmp( "--backlight", argv[ nIdx ] ) )
            {
                const char * pchBacklightId = getArgument( nIdx + 1, argc, argv );
                const char * pchDuration = getArgument( nIdx + 2, argc, argv );
                const char * pchRed = getArgument( nIdx + 3, argc, argv );
                const char * pchGreen = getArgument( nIdx + 4, argc, argv );
                const char * pchBlue = getArgument( nIdx + 5, argc, argv );
                const char * pchWhite = getArgument( nIdx + 6, argc, argv );

                if ( pchBacklightId && pchDuration && pchRed && pchGreen && pchBlue && pchWhite )
                {
                    backlightParameter_t bl;
                    bl.nId = atoi( pchBacklightId );
                    bl.nDur = atoi( pchDuration );
                    bl.nRR = atoi( pchRed );
                    bl.nGG = atoi( pchGreen );
                    bl.nBB = atoi( pchBlue );
                    bl.nWW = atoi( pchWhite );
                    backlight.push_back( bl );
                    nIdx += 6;
                }
                else
                {
                    nRes = ERR_INSUFFICIENT_PARAMETER;
                    nInvalidArgIndex = nIdx;
                }
            }

            // --upload-firmware <filepath>
            else if ( !strcmp( "--upload-firmware", argv[ nIdx ] ) )
            {
                const char * pchFilename = getArgument( nIdx + 1, argc, argv );
                if ( pchFilename )
                {
                    firmwareUpload.strFirmwareFilename = pchFilename;
                    nIdx += 1;
                }
                else
                {
                    nRes = ERR_INSUFFICIENT_PARAMETER;
                    nInvalidArgIndex = nIdx;
                }
            }

            else if ( !strcmp( "--upload-setup", argv[ nIdx ] ) )
            {
                const char * pchFilename = getArgument( nIdx + 1, argc, argv );
                if ( pchFilename )
                {
                    setupUpload.strSetupFilename = pchFilename;
                    nIdx += 1;
                }
                else
                {
                    nRes = ERR_INSUFFICIENT_PARAMETER;
                    nInvalidArgIndex = nIdx;
                }
            }

            else if ( !strcmp( "--upload-setup-list", argv[ nIdx ] ) )
            {
                const char * pchListname = getArgument( nIdx + 1, argc, argv );
                const char * pchFilename = getArgument( nIdx + 2, argc, argv );
                if ( pchListname && pchFilename )
                {
                    listParameter_t lp;
                    lp.nListMode = 1; // 1 == Setup-Liste, 2 == Zutrittskontroll-Liste
                    lp.strListName = pchListname;
                    lp.strListFilename = pchFilename;
                    listParameter.push_back( lp );
                    nIdx += 2;
                }
                else
                {
                    nRes = ERR_INSUFFICIENT_PARAMETER;
                    nInvalidArgIndex = nIdx;
                }
            }

            else if ( !strcmp( "--upload-access-control-list", argv[ nIdx ] ) )
            {
                const char * pchListname = getArgument( nIdx + 1, argc, argv );
                const char * pchFilename = getArgument( nIdx + 2, argc, argv );
                if ( pchListname && pchFilename )
                {
                    listParameter_t lp;
                    lp.nListMode = 2; // 1 == Setup-Liste, 2 == Zutrittskontroll-Liste
                    lp.strListName = pchListname;
                    lp.strListFilename = pchFilename;
                    listParameter.push_back( lp );
                    nIdx += 2;
                }
                else
                {
                    nRes = ERR_INSUFFICIENT_PARAMETER;
                    nInvalidArgIndex = nIdx;
                }
            }

            else if ( !strcmp( "--read-setup-list", argv[ nIdx ] ) )
            {
                const char * pchListname = getArgument( nIdx + 1, argc, argv );
                if ( pchListname  )
                {
                    listParameter_t lp;
                    lp.nListMode = 1; // 1 == Setup-Liste, 2 == Zutrittskontroll-Liste
                    lp.strListName = pchListname;
                    listenLesenParameter.push_back( lp );
                    nIdx += 1;
                }
                else
                {
                    nRes = ERR_INSUFFICIENT_PARAMETER;
                    nInvalidArgIndex = nIdx;
                }
            }

            else if ( !strcmp( "--read-access-control-list", argv[ nIdx ] ) )
            {
                const char * pchListname = getArgument( nIdx + 1, argc, argv );
                if ( pchListname )
                {
                    listParameter_t lp;
                    lp.nListMode = 2; // 1 == Setup-Liste, 2 == Zutrittskontroll-Liste
                    lp.strListName = pchListname;
                    listenLesenParameter.push_back( lp );
                    nIdx += 1;
                }
                else
                {
                    nRes = ERR_INSUFFICIENT_PARAMETER;
                    nInvalidArgIndex = nIdx;
                }
            }

            else if ( !strcmp( "--read-sys-var", argv[ nIdx ] ) )
            {
                const char * pchName = getArgument( nIdx + 1, argc, argv );
                if ( pchName )
                {
                    vecReadSystemVariables.push_back( pchName );
                    nIdx += 1;
                }
                else
                {
                    nRes = ERR_INSUFFICIENT_PARAMETER;
                    nInvalidArgIndex = nIdx;
                }
            }

            else if ( !strcmp( "--write-sys-var", argv[ nIdx ] ) )
            {
                const char * pchName = getArgument( nIdx + 1, argc, argv );
                const char * pchValue = getArgument( nIdx + 2, argc, argv );
                if ( pchName && pchValue)
                {
                    vecWriteSystemVariables.push_back( std::make_pair( pchName, pchValue ) );
                    nIdx += 2;
                }
                else
                {
                    nRes = ERR_INSUFFICIENT_PARAMETER;
                    nInvalidArgIndex = nIdx;
                }
            }

            else
            {
                nInvalidArgIndex = nIdx;
                break;
            }

            // Fehler bei der Parameter-Verarbeitung?
            if ( nRes != ERR_NO_ERROR )
            {
                nInvalidArgIndex = nIdx;
                break;
            }
        }
    }

    char version[64];                               // Puffer zum Halten von Versionen.
    memset( version, 0, sizeof( version ) );        // Initialisieren des Puffers zur Version.

    if ( nRes == ERR_NO_ERROR )
    {
        if ( !strOutputFilename.empty() )
        {
            pOutputFile = fopen( strOutputFilename.c_str(), "wb" );
            if ( !pOutputFile )
            {
                my_printf( "Failed to create output file '%s'.\n", strOutputFilename.c_str() );
                nRes = ERR_FILE_CREATE_ERROR;
            }
        }
    }

    if ( nRes == ERR_NO_ERROR )
    {
        int length = sizeof( version );    // Variable für eine Längenangabe.

        // Abrufen der aktuell geladenen DFCom-Version.
        if ( !DFCVersionLibrary( version, length ) )
        {
            my_printf( "DFCom Library failed to load.\n" );
            nRes = ERR_DFCOM_LOAD_FAILED;
        }

        if ( bVerbose )
        {
            my_printf( "DLL Version %s\n", version );
        }
    }

    if ( bHelpRequested || ( nInvalidArgIndex > 0 ) )
    {
        //         0         10        20        30        40        50        60        70        80
        my_printf( "Synopsis %s [...]\n\n", argv[ 0 ] );
        my_printf( "  General parameters:\n" );
        my_printf( "    --dump                        Print current setting parameters to console\n" );
        my_printf( "    --help                        Show this help\n" );
        my_printf( "    --verbose                     Show steps of data processing step by step\n" );
        my_printf( "    --output <filename>           Write application out to <filename> in\n" );
        my_printf( "                                  addition to stdout.\n" );
        my_printf( "\n" );
        my_printf( "  Device definition:\n" );
        my_printf( "    --serial-device <com-port>    Use device connected to COM port <com-port>\n" );
        my_printf( "    --baud-rate <baudrate>        Set Baud Rate for COM attached device.\n" );
        my_printf( "    --tcp-device <address> <port> Use TCP/IP connected passive mode device\n" );
        my_printf( "    --communication-timeout <time ms>\n" );
        my_printf( "                                  Set Timeout for TCP/IP based communication\n" );
        my_printf( "\n" );
        my_printf( "  Device Maintenance:\n" );
        my_printf( "    --print-lists                 Prints the lists present on the device\n" );
        my_printf( "                                  (including access control lists)\n" );
        my_printf( "    --read-device-log             Read the device' log and print it to stdout\n" );
        my_printf( "    --read-records                Reads the data records from the device and\n" );
        my_printf( "                                  prints them to stdout.\n" );
        my_printf( "    --read-record                 Reads a single data record from the device\n" );
        my_printf( "                                  and prints it to stdout.\n" );
        my_printf( "    --read-sys-var <name>         Reads the current value of a device's system\n" );
        my_printf( "                                  variable.\n" );
        my_printf( "    --read-setup-list <name>\n    Reads a setup lists and prints its content to\n" );
        my_printf( "                                  stdout.\n" );
        my_printf( "    --read-access-control-list <name>\n" );
        my_printf( "                                  Reads an access control list and prints its\n" );
        my_printf( "                                  content to stdout.\n" );
        my_printf( "\n" );
        my_printf( "  Action definition:\n" );
        my_printf( "    --backlight <id> <duration> <red> <green> <blue> <white>\n" );
        my_printf( "                                  Set the color of a device backlight for a\n" );
        my_printf( "                                  specified duration\n" );
        my_printf( "    --upload-firmware <filepath>  Uploads a firmware to the device\n" );
        my_printf( "    --upload-setup <filepath>     Uploads a setup to the device\n" );
        my_printf( "    --upload-setup-list <name> <filepath>\n" );
        my_printf( "                                  Uploads a list data file at <filepath> as\n" );
        my_printf( "                                  list <name> to the device. The list <name>\n" );
        my_printf( "                                  Uploads a list data file at <filepath> as\n" );
        my_printf( "                                  to be a 'normal' setup defined list.\n" );
        my_printf( "    --upload-access-control-list <name> <filepath>\n" );
        my_printf( "                                  Uploads a list data file at <filepath> as\n" );
        my_printf( "                                  list <name> to the device. The list <name>\n" );
        my_printf( "                                  Uploads a list data file at <filepath> as\n" );
        my_printf( "                                  to be one of the predefined access control\n" );
        my_printf( "                                  lists.\n" );
        my_printf( "    --write-sys-var <name> <value>\n" );
        my_printf( "                                  Sets the value of a system variable.\n" );

        if ( nInvalidArgIndex > 0 )
        {
            my_printf( "\n* Invalid argument %s at position %i.\n", argv[ nInvalidArgIndex ], nInvalidArgIndex );
            nRes = ERR_INVALID_ARGUMENT;
        }
    }

    if ( nRes == ERR_NO_ERROR )
    {
        if ( !DFCComOpenIV( connection_id, 0, connection_type, connection_string, connection_value, communication_read_timeout ) )
        {
            my_printf( "Failed to open connection to device %s %i %i (TYPE %i)\n", connection_string, connection_value, communication_read_timeout, connection_type );
            nRes = ERR_DFCOM_OPEN_FAILED;
        }
    }

    int nErrCode = 0;

    if ( nRes == ERR_NO_ERROR )
    {
        if ( bReadRecords || bReadSingleRecord )
        {
            // Read data records from the devicec

            if ( !DFCLoadDatensatzbeschreibung( connection_id, device_address, &nErrCode ) )
            {
                my_printf( "ERROR DLL-%i reading data record definition from device\n", nErrCode );
                nRes = ERR_DFCOM_DFCLOADRECORDDESCRIPTION_FAILED;
            }

            int nCnt = 0;

            while ( nRes == ERR_NO_ERROR )
            {
                unsigned char buffer[ 300 ];
                int nBufferSz = sizeof( buffer );
                memset( buffer, 0, sizeof( buffer ) );

                int nRR = DFCReadRecord( connection_id, device_address, buffer, &nBufferSz, &nErrCode );

                if ( nRR == 0 )
                {
                    // No datarecords on device
                    break;
                }
                else if ( nRR >= 1 && nRR <= 4 )
                {
                    // on/offline data record
                    my_printf( "DATA RECORD:\n" );
                    printHexMem( (const char*) buffer, nBufferSz );
                    my_printf( "\n" );
                    ++nCnt;

                    if ( !DFCQuitRecord( connection_id, device_address, &nErrCode ) )
                    {
                        my_printf( "ERROR quit record failed with error code DLL-%i\n", nErrCode );
                        nRes = ERR_DFCOM_DFCQUITRECORD_FAILED;
                    }
                }
                else
                {
                    my_printf( "ERROR read record failed with error code DLL-%i\n", nErrCode );
                    nRes = ERR_DFCOM_DFCREADRECORD_FAILED;
                }

                if ( bReadSingleRecord && !bReadRecords )
                    break;
            }

            my_printf( "%i data records read from device.\n", nCnt );
        }
    }

    if ( nRes == ERR_NO_ERROR )
    {
        for ( auto strName : vecReadSystemVariables )
        {
            char buffer[ 1024 ];
            memset( buffer, 0, sizeof( buffer ) );

            if ( !DFCGetGlobVar( connection_id, device_address,
                                 strName.c_str(), 2,
                                 buffer, sizeof( buffer ) - 1, &nErrCode ) )
            {
                my_printf( "ERROR read system variable '%s' failed with error code DLL-%i\n", strName.c_str(), nErrCode );
                nRes = ERR_DFCOM_DFCREADSYSVAR_FAILED;
            }
            else
            {
                my_printf( "SYSVAR: '%s' has value: ", strName.c_str() );
                printDfString( buffer );
                my_printf( "\n" );
            }
        }
    }

    if ( nRes == ERR_NO_ERROR )
    {
        for ( auto kvp : vecWriteSystemVariables )
        {
            if ( !DFCSetGlobVar( connection_id, device_address,
                                 kvp.first.c_str(), 2, kvp.second.c_str(),
                                 &nErrCode ) )
            {
                my_printf( "ERROR write system variable '%s' failed with error code DLL-%i\n", kvp.first.c_str(), nErrCode );
                nRes = ERR_DFCOM_DFCREADSYSVAR_FAILED;
            }
        }
    }

    if ( nRes == ERR_NO_ERROR )
    {
        for ( auto bl : backlight )
        {
            char message[243];                       // Puffer für eine zu sendende Nachricht. Maximal 242 Zeichen Länge.
            memset( message, 0, sizeof( message ) ); // Initialisieren des Puffers für die Nachricht.

            // Nachricht für Display aufbereiten.
            const int length = sprintf( message, "df_backlight=%i,%i,%i,%i,%i,%i", bl.nId, bl.nDur, bl.nRR, bl.nGG, bl.nBB, bl.nWW ) + 1; // Inkl. Nullterminierung.

            unsigned char visible_time   = 1; // Anzeigedauer in Sekunden.
            unsigned char do_action      = 1; // 1 für Verarbeitung von HTTP-Telegrammen
            unsigned char audible_signal = 0; // Akkustisches Signal. 1 = OK, 2 = Error, etc...

            // Nachricht zum Gerät senden.
            if ( !DFCComSendMessage( connection_id, device_address,
                                     visible_time, do_action, audible_signal, message, length ) )
            {
                my_printf( "ERROR: Nachricht konnte nicht an das Geraet gesendet werden.\n" );
                nRes = ERR_DFCOM_SEND_MESSAGE;
            }
        }
    }

    if ( nRes == ERR_NO_ERROR )
    {
        if ( !firmwareUpload.strFirmwareFilename.empty() )
        {
            std::string strDfzFilename = firmwareUpload.strFirmwareFilename;
#ifdef _WIN32
            if ( !DFCUpload( connection_id, device_address, strDfzFilename.c_str(), &nErrCode ) )
            {
                my_printf( "ERROR %i during firmware upload\n", nErrCode );
                nRes = ERR_DFCOM_DFCUPLOAD_FAILED;
            }

#else
            int nErrCode = ERR_NOT_IMPLEMENTED;

#if 0
#!/bin/bash

fn=$1

tmpdir=`mktemp -d`
res=$?
echo $res $tmpdir

unzip -q $fn -d $tmpdir

du -sh $tmpdir

rm -rf $tmpdir
#endif

#warning "Unpack DFZ and supply directory"


            // TODO Verzeichnis entpacken und Pfad zum Verzeichnis an DFCUpload übergeben



            // int nRes = system( ( "unzip " + firmwareUpload.strFilename ).c_str() );
            // if ( nRes != 0 )
            // {
            //     my_printf( "Unpacking %s failed with error code %i (from unzip)\n", firmwareUpload.strFilename.c_str(), nRes );
            //     nRes = ERR_DFCOM_UNZIP_DFC_FAILED;
            // }
#endif
            // Assume the DFZ parameter points to the directory, where the data has been extracted to
            if ( !DFCUpload( connection_id, device_address, strDfzFilename.c_str(), &nErrCode ) )
            {
                my_printf( "ERROR %i during firmware upload\n", nErrCode );
                nRes = ERR_DFCOM_DFCUPLOAD_FAILED;
            }

        }
    }

    // Setup übertragen
    if ( nRes == ERR_NO_ERROR )
    {
        if ( !setupUpload.strSetupFilename.empty() )
        {
            int nErrCode = 0;
            if ( !DFCSetupLaden( connection_id, device_address, setupUpload.strSetupFilename.c_str(), &nErrCode ) )
            {
                my_printf( "DLL-ERROR %i during setup upload\n", nErrCode );
                nRes = ERR_DFCOM_DFCSETUPLADEN_FAILED;
            }
        }
    }

    // Listendaten übertragen
    if ( nRes == ERR_NO_ERROR )
    {
        DFCClrListenBuffer( connection_id );
        for ( int nAc2ListId = 0; nAc2ListId < 8; ++nAc2ListId )
        {
            DFCClearEntrance2ListBuffer( connection_id, nAc2ListId );
        }

        bool bHaveSetupLists = false;
        bool bHaveAc2Lists = false;
        for ( auto lp : listParameter )
        {
            nRes = prepareList( connection_id, device_address, lp );

            bHaveSetupLists |= ( lp.nListMode == 1 );
            bHaveAc2Lists |= ( lp.nListMode == 2 );

            if ( nRes != ERR_NO_ERROR )
                break;
        }

        if ( nRes == ERR_NO_ERROR )
        {
            int nErrCode = 0;
            int nResLoadList = 0;

            if ( bHaveAc2Lists )
            {
                nResLoadList = DFCLoadEntrance2List( connection_id, device_address, -1, &nErrCode );

                if ( nResLoadList == 0 )
                {
                    my_printf( "ERROR %i during AC2 list upload\n", nErrCode );
                    nRes = ERR_DFCOM_DFCLISTLOAD_FAILED;
                }
            }

            if ( bHaveSetupLists )
            {
                nResLoadList = DFCLoadListen( connection_id, device_address, &nErrCode );

                if ( nResLoadList == 0 )
                {
                    my_printf( "ERROR %i during setup list upload\n", nErrCode );
                    nRes = ERR_DFCOM_DFCLISTLOAD_FAILED;
                }
            }
        }
    }

    // Listendaten lesen
    for ( auto lp : listenLesenParameter )
    {
        int tableHandle = -1;
        if ( nRes == ERR_NO_ERROR )
        {
            std::string strTableName = ( lp.nListMode == 1 ? "LIST." : "ACCESS." ) + lp.strListName;
            int nResTableOpen = DFCTableOpen( connection_id, device_address, strTableName.c_str(), 0, &tableHandle );
            if ( nResTableOpen == 0 )
            {
                my_printf( "ERROR %i while opening table '%s'\n", DFCGetLastErrorNumber( connection_id, device_address ), strTableName.c_str() );
                nRes = ERR_DFCOM_DFCTABLE_OPEN;
            }

            int currentCount = -1, unsortedCount = -1, deletedCount = -1;
            if ( nRes == ERR_NO_ERROR )
            {
                int nResGetRowCount = DFCTableGetRowCount( connection_id, device_address, tableHandle, &currentCount, &unsortedCount, &deletedCount );
                if ( nResGetRowCount == 0 )
                {
                    my_printf( "ERROR %i while getting counts frrom table '%s'\n", DFCGetLastErrorNumber( connection_id, device_address ), strTableName.c_str() );
                    nRes = ERR_DFCOM_DFCTABLE_GETCOUNTS;
                }
            }
            if ( nRes == ERR_NO_ERROR )
            {
                my_printf( "Table '%s' contains %i sorted, %i unsorted and %i deleted rows\n", strTableName.c_str(), currentCount, unsortedCount, deletedCount );

                my_printf( "Reading table data...\n" );
                char data[ 255 ];
                while ( true )
                {
                    memset( data, 0, sizeof( data ) );
                    int nSize = sizeof( data ) - 1;
                    int nResReadRow = DFCTableGetCurrentRowData( connection_id, device_address, tableHandle, 1, 59, data, &nSize );
                    if ( nResReadRow == 0 )
                        break;
                    my_printf( "ROW DATA '%s'\n", data );
                }
                my_printf( "Reading table data... done\n" );
            }


            DFCTableClose( connection_id, device_address,  tableHandle );
        }
    }

    if ( bPrintListen )
    {
        DFCClrListenBuffer( connection_id );

        int nErrCode = -1;
        int nLLRes = DFCLoadListenbeschreibung( connection_id, device_address, &nErrCode );
        if ( nLLRes == 0 )
        {
           my_printf( "FEHLER while reading device's list definition. DLL-Error-Code %i\n", nErrCode );
           nRes = ERR_DFCOM_DFCLISTLOAD_FAILED;
        }

        if ( nRes == ERR_NO_ERROR )
        {
            const int nSetupListCnt = DFCListBCnt( connection_id );
            my_printf( "Number of setup lists defined: %i\n", nSetupListCnt );

            for ( int nIdx = 0; nIdx < nSetupListCnt; ++nIdx )
            {
                printListe( connection_id, nIdx );
            }
            for ( int nIdx = 200; nIdx <= 207; ++nIdx )
            {
                printListe( connection_id, nIdx );
            }

            my_printf( "Attention:\n" );
            my_printf( "* The list field width includes one padding byte at the end.\n" );
            my_printf( "* Access control lists are printed without checking, if access control is\n" );
            my_printf( "  activated at the specific device.\n" );
        }
    }

    if ( bReadLog )
    {
        unsigned char arr[ 2000 ];
        while( true )
        {
            memset( arr, 0, sizeof( arr ) );
            int nSz = sizeof( arr ) - 1;
            if ( !DFCSystemReadRecord( connection_id, device_address, arr, &nSz, &nErrCode ) )
            {
                // End of log?
                if ( nErrCode == 0 )
                    break;

                my_printf( "DLL-ERROR %i during read log record\n", nErrCode );
                nRes = ERR_DFCOM_DFCREADSYSRECORD_FAILED;
                break;
            }
            else
            {
                my_printf( "%s", arr );
            }


            if ( !DFCSystemQuitRecord( connection_id, device_address, &nErrCode ) )
            {
                my_printf( "DLL-ERROR %i during quit log record\n", nErrCode );
                nRes = ERR_DFCOM_DFCQUITSYSRECORD_FAILED;
                break;
            }
        }
    }

    DFCComClose( connection_id );

    if ( bDump )
    {
        my_printf( "===> DUMP\n" );
        my_printf( "  DFCom Version: %s\n", version );

        char arr[ 100 ];
        memset( arr, 0, sizeof( arr ) );
        int nErrCode = 0;
        DFCGetVersionFirmware( connection_id, device_address, arr, &nErrCode );
        my_printf( "  Device FW      %s\n", arr );

        my_printf( "  Dump:          %i\n", bDump );
        my_printf( "  Help:          %i\n", bHelpRequested );
        my_printf( "  Backlights:    %i\n", backlight.size() );
        for ( auto bl : backlight )
        {
            my_printf( "    Backlight:   ID %i DURATION %i sec. RR %i GG %i BB %i WW %i\n",
                    bl.nId, bl.nDur, bl.nRR, bl.nGG, bl.nBB, bl.nWW );
        }
        my_printf( "  Firmware:      '%s'\n", firmwareUpload.strFirmwareFilename.c_str() );
        my_printf( "  Setup:         '%s'\n", setupUpload.strSetupFilename.c_str() );
        my_printf( "  Lists:         %i\n", listParameter.size() );
        for ( auto lp : listParameter )
        {
            my_printf( "    List:        TYPE %i NAME '%s' FILENAME '%s'\n", lp.nListMode, lp.strListName.c_str(), lp.strListFilename.c_str() );
        }
        my_printf( "\n" );
        my_printf( "  RESULT CODE:   %i\n", nRes );
        my_printf( "<=== DUMP\n" );

    }

    if ( pOutputFile )
    {
        fclose( pOutputFile );
    }

    return nRes;
}
