/*
    \brief Deklarationen der Funktionen für die Arbeit mit Fingertemplates.

    \author Bernd Ottmann
    \date   21.02.2003
*/
#pragma once
#ifndef DATAFOX_FINGERPRINT_H
#ifndef DOXYGEN_SHOULD_SKIP_THIS
#define DATAFOX_FINGERPRINT_H
#endif

#include "df_defines.h"

#if (DF_API_VERSION >= 0x04010400)

/*!
    \brief Anfügen eines Fingertemplates an die Datentabelle des Fingerprints. Siehe auch \ref dfc_fingerprint.

    \ingroup fingerprint

    \export{_DFCFingerprintAppendRecord@24, DFCFingerprintAppendRecord}

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[in] templateType Templatetyp der übergeben wird.
    <table class="paramvalues">
        <tr><th>Wert</th><th>Bedeutung</th></tr>
        <tr><td>0</td><td>\ref dfc_fingerprint_format_DINV66400 "DIN V66400" (161 Byte)</td></tr>
        <tr><td>1</td><td>\ref dfc_fingerprint_format_IdencomCompact "Idencom-Compact" (216 Byte)</td></tr>
        <tr><td>2</td><td>\ref dfc_fingerprint_format_IdencomStandard "Idencom-Standard" (561)</td></tr>
        <tr><td>3</td><td>\ref dfc_fingerprint_format_Saturn "Saturn" (635 Byte)</td></tr>
    </table>
    \param[in] buffer Datenarray des Templates.
    \param[in] bufferSize Die Länge des Datenarrays.
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.

    \since 04.01.04 enthalten
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCFingerprintAppendRecord( int connectionNumber, int deviceAddress, int templateType, const unsigned char *buffer, int bufferSize, int *errorNumber );

/*!
    \brief Lesen eines Fingertemplates.

    Über die eindeutige Kombination aus Personal-Id (PID)/ Finger-Id (FID) können mit dieser Funktion die
    entsprechenden Templatedaten ermittelt werden.

    \ingroup fingerprint

    \export{_DFCFingerprintGetRecord@32, DFCFingerprintGetRecord}

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[in] templateType Templatetyp wie bei DFCFingerprintAppendRecord() beschrieben.
    \param[in] pid Personal-Id des Templates.
    \param[in] fid Finger-Id des Templates.
    \param[out] buffer Datenarray für das Template.
    \param[in,out] bufferSize Als \c in die Länge des Datenarrays von \a buffer, als \c out die übergebene Anzahl Datenbytes.
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.

    \since 04.01.04 enthalten
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCFingerprintGetRecord( int connectionNumber, int deviceAddress, int templateType, int pid, int fid, unsigned char *buffer, int *bufferSize, int *errorNumber );

/*!
    \brief Löschen von Fingertemplates.

    Sie können einzelne Templates durch Angabe des entsprechenden Personal-Id, Finger-Id Pärchens löschen.
    Es ist auch möglich durch Angabe einer Personal-Id und einer Finger-Id = 0, alle Fingertemplates zu dieser Personal-Id zu löschen.
    Wird eine Personal-Id = 0 und Finger-Id = 0 angegeben, werden alle Fingertemplates des Moduls gelöscht.

    \ingroup fingerprint

    \export{_DFCFingerprintDeleteRecord@20, DFCFingerprintDeleteRecord}

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[in] pid Personal-Id des Templates.
    \param[in] fid Finger-Id des Templates.
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.

    \since 04.01.04 enthalten
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCFingerprintDeleteRecord( int connectionNumber, int deviceAddress, int pid, int fid, int *errorNumber );

#endif

#if (DF_API_VERSION >= 0x04010700)

/*!
    \brief Erzeugen einer Systemliste (FINGERPRINT.IDLIST) aller enthaltener Personal-Id's und zugehöriger Finger-Id's.

    Es wird eine Liste \c FINGERPRINT.IDLIST erzeugt mit 2 Spalten \c PID und \c FID die alle auf dem Modul enthaltenen
    Personal-Id und Finger-Id's der Templates enthält. Mit diesen Daten können dann mit DFCFingerprintGetRecord()
    alle Fingertemplates ausgelesen werden.\n
    Auf die erstellte Liste können Sie mit den Funktionen für das Listeneditieren wie DFCTableOpen() zugreifen.

    \ingroup fingerprint

    \export{_DFCFingerprintList@16, DFCFingerprintList}

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[in] flags Flags für zusätzliche Angaben.
    <table class="paramvalues">
        <tr><th>Bitnummer</th><th>Wert</th><th>Bedeutung</th></tr>
        <tr><td>1</td><td>1</td><td>Wird dieses Flag gesetzt, liefert die Funktion die Anzahl der Datensätze der Liste
            zurück. Wenn keine Datensätze vorliegen bricht die Funktion mit einem Fehler
            ab. Eine Datensatzanzahl von 0 gibt es somit nicht.</td></tr>
        <tr><td>...</td><td></td><td>Nicht verwendet. Bitte immer 0 übergeben.</td></tr>
    </table>
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.
    \retval >0 Bei Angabe des \a flags 1 wird die Anzahl der Listensätze geliefert.

    \since 04.01.07 enthalten
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCFingerprintList( int connectionNumber, int deviceAddress, int flags, int *errorNumber );

/*!
    \brief Backupdatei mit allen Fingertemplates erzeugen.

    Es werden alle Fingertemplates ausgelesen und in die angegebene Datei gespeichert. Durch Aufruf von
    DFCFingerprintRestore() kann die Datei dann ggf. auf andere Terminals übertragen werden.

    \ingroup fingerprint

    \export{_DFCFingerprintBackup@20, DFCFingerprintBackup}

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[in] filePath Dateiname der zu erstellenden Backupdatei.
    \param[in] flags Reservierter Parameter. Standardwert 0.
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.

    \since 04.01.07 enthalten
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCFingerprintBackup( int connectionNumber, int deviceAddress, const char *filePath, int flags, int *errorNumber );

/*!
    \brief Backupdatei mit Fingertemplates auf ein Gerät übertragen.

    Es wird eine mit DFCFingerprintRestore() erstellte  Backupdatei auf das Gerät übertragen.

    \ingroup fingerprint

    \export{_DFCFingerprintRestore@20, DFCFingerprintRestore}

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[in] filePath Dateiname der zu übertragenden Backupdatei.
    \param[in] flags Reservierter Parameter. Standardwert 0.
    <table class="paramvalues">
        <tr><th>Bitnummer</th><th>Wert</th><th>Bedeutung</th></tr>
        <tr><td>1</td><td>1</td><td>Vor der Übertragung werden durch Aufruf der Löschfunktion alle auf dem
            Modul enthaltenden Templates gelöscht.</td></tr>
        <tr><td>...</td><td></td><td>Nicht verwendet. Bitte immer 0 übergeben.</td></tr>
    </table>
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.

    \since 04.01.07 enthalten
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCFingerprintRestore( int connectionNumber, int deviceAddress, const char *filePath, int flags, int *errorNumber );

#endif

#endif
