/****************************************************************************
**
** Copyright (C) Datafox GmbH.
** All rights reserved.
** Contact: Datafox GmbH (develop@datafox.de)
**
** This file is part of the DFComDLL module of the Datafox SDK.
**
****************************************************************************/

#include "datafox/communication/module_table.h"
#include <stdio.h>
#include <cstring>

// ##################################################################
/**
 * @brief Initialisieren der Membervariablen.
 *
 * @param id Nummer der Kanalklasse fuer z.B. Trace.
 */
Datafox::Communication::ModuleTable::ModuleTable( int32_t connectionNumber, uint8_t deviceAddress )
    : ModuleApplication( connectionNumber, deviceAddress )
{
    trace( "%s(%i):INFO(3): %s [ %i, %u ]\n", __FILE__, __LINE__, __func__, connectionNumber, deviceAddress );
}

// ##################################################################
/**
 * @brief Destruktor.
 *
 */
Datafox::Communication::ModuleTable::~ModuleTable()
{
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
void Datafox::Communication::ModuleTable::errorModule( uint16_t module, uint16_t error, std::string &text )
{
    trace( "%s(%i):INFO(3): %s [ %u, %u, 0x%08X ]\n", __FILE__, __LINE__, __func__, module, error, text.c_str() );

    // Ist IV_MODULE_TABLE ?
    if ( module == IV_MODULE_TABLE )
    {

    }

    // Ermittlung des Fehlertextes an Basisklasse abgeben.
    ModuleApplication::errorModule( module, error, text );
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
void Datafox::Communication::ModuleTable::errorFunction( uint16_t module, uint16_t func, uint16_t error, std::string &text )
{
    trace( "%s(%i):INFO(3): %s [ %u, %u, %u, 0x%08X ]\n", __FILE__, __LINE__, __func__, module, func, error, text.c_str() );

    if ( module == IV_MODULE_TABLE )
    {
        // Sonderfehler für Passwortschutz abhandeln.
        if ( error == 28 )
        {
            // Die Modulfunktionen liefern den Fehlercode 28, wenn kein oder ein falsches Passwort verwendet wurde.
            // Es ist in diesem Fall zuerst das richtige Passwort für die Kommunikation erforderlich.
            text = boost::str( boost::format( "( %u ): Korrektes Passwort für die Kommunikation erforderlich." ) % error );
            return;
        }
        else if ( error == 29 )
        {
            // Die Modulfunktionen liefern den Fehlercode 29, wenn 3 mal ein falsches Passwort verwendet wurde.
            // Die Befehlsausführung ist danach für 60 Sekunden blockiert.
            text = boost::str( boost::format( "( %u ): Gerät ist aufgrund von fehlerhaften Passworteingaben für 60 Sekunden gesperrt." ) % error );
            return;
        }

        switch ( func )
        {
            case IV_FUNCTION_TABLE_GET:
            case IV_FUNCTION_TABLE_GET_COLUMN:
                switch ( error )
                {
                    case 1: text = boost::str( boost::format( "( %u ): Der Majorbestandteil des Tabellennamens ist unbekannt." ) % error ); break;
                    case 2: text = boost::str( boost::format( "( %u ): Der Minorbestandteil des Tabellennamens ist unbekannt." ) % error ); break;
                    case 3: text = boost::str( boost::format( "( %u ): Der Subminorbestandteil (Feldname) des Tabellenfeldnamens ist unbekannt." ) % error ); break;
                    default: text = boost::str( boost::format( "( %u ): Nicht eingetragener Funktionsfehler!" ) % error ); break;
                }
                break;

            case IV_FUNCTION_TABLE_OPEN:
                switch ( error )
                {
                    case 1: text = boost::str( boost::format( "( %u ): Der Majorbestandteil des Tabellennamens ist unbekannt." ) % error ); break;
                    case 2: text = boost::str( boost::format( "( %u ): Der Minorbestandteil des Tabellennamens ist unbekannt." ) % error ); break;
                        // Standardmaessig ist die Option mit 0 angegeben. Ein gesetztes Bit, dessen Bedeutung unklar ist, ergibt diesen Fehler.
                    case 3: text = boost::str( boost::format( "( %u ): Unbekannte Option gesetzt." ) % error ); break;
                        // Wenn ein reservierter Parameter angegeben wurde.
                    case 4: text = boost::str( boost::format( "( %u ): Der angegebene Reservierte Parameter wird nicht unterstützt." ) % error ); break;
                        // Die Tabelle ist bereits mit schreib - Recht geoffnet worden.
                    //case 5: text = boost::str( boost::format("( %u ): Die Tabelle kann nur lesend geoeffnet werden." ) % error ); break;
                        // Die Tabelle ist bereits mit Exclusive - Recht geoffnet worden.
                    //case 6: text = boost::str( boost::format("( %u ): Die Tabelle ist z. Z. gesperrt." ) % error ); break;
                        // Die Nutzung der Tabelle wurde aufgrund von Resourcenmangel versagt. (Zu oft geoeffnet worden usw..)
                    //case 7: text = boost::str( boost::format("( %u ): Die Nutzung der Tabelle wurde aufgrund von Resourcenmangel versagt." ) % error ); break;
                    default: text = boost::str( boost::format( "( %u ): Nicht eingetragener Funktionsfehler!" ) % error ); break;
                }
                break;

            case IV_FUNCTION_TABLE_CLOSE:
                switch ( error )
                {
                    // Wenn versucht wird ein Handle zu schliessen der keiner offenen Tabelle zugehoerig ist.
                    case 1: text = boost::str( boost::format( "( %u ): Angegebener Handle war nicht geoffnet." ) % error ); break;
                        // Liste konnte nicht geschlossen werden.
                    case 2: text = boost::str( boost::format( "( %u ): Tabelle konnte nicht geschlossen werden." ) % error ); break;
                    default: text = boost::str( boost::format( "( %u ): Nicht eingetragener Funktionsfehler!" ) % error ); break;
                }
                break;

            case IV_FUNCTION_TABLE_SET_FILTER:
            case IV_FUNCTION_TABLE_SET_FILTER_OK:
                switch ( error )
                {
                    // Wenn der angegebene Handle nicht existiert.
                    case 1: text = boost::str( boost::format( "( %u ): Angegebener Handle ist nicht geoffnet." ) % error ); break;
                        // Ungueltiger Selektionsspaltenanzahl.
                    case 2: text = boost::str( boost::format( "( %u ): Falsche Anzahl von Selektionsspalten." ) % error ); break;
                        // Nach setzen des Filterkriteriums -> Kein Datensatz
                    case 3: text = boost::str( boost::format( "( %u ): Filterkriterium liefert kein Ergebnis." ) % error ); break;
                        // Spaltenname des Filter unbekannt.
                    case 4: text = boost::str( boost::format( "( %u ): Unbekannter Spaltenname." ) % error ); break;
                        // Unbekannter (nicht unterstuetzter) Filteroperator.
                    case 5: text = boost::str( boost::format( "( %u ): Unbekannter Filteroperator." ) % error ); break;
                        // Fehlender in Anfuehrungszeichenpaar des zu filterndem Spaltenwerts.
                        // Keine vorhanden, unterschiedliches ein/aus '" oder fehlender ein oder Ausgang.
                    case 6: text = boost::str( boost::format( "( %u ): Fehler in Anfuehrungszeichenpaar des zu filterndem Spaltenwertes." ) % error ); break;
                        // Zu viele Parameter angegeben.
                    case 7: text = boost::str( boost::format( "( %u ): Zu viele Parameter angegeben." ) % error ); break;

                    default: text = boost::str( boost::format( "( %u ): Nicht eingetragener Funktionsfehler!" ) % error ); break;
                }
                break;

            case IV_FUNCTION_TABLE_GET_FILTER:
            case IV_FUNCTION_TABLE_REMOVE_FILTER:
                switch ( error )
                {
                    // Wenn der angegebene Handle nicht existiert.
                    case 1: text = boost::str( boost::format( "( %u ): Angegebener Handle ist nicht geoffnet." ) % error ); break;
                        // Auf der angegebenen Tabelle ist kein Filter gesetzt.
                    //case 2:text = boost::str( boost::format("( %u ): Kein Filter bei Tabelle angegeben." ) % error ); break;

                    default: text = boost::str( boost::format( "( %u ): Nicht eingetragener Funktionsfehler!" ) % error ); break;
                }
                break;

            case IV_FUNCTION_TABLE_SET_CURRENT_ROW:
                switch ( error )
                {
                    // Wenn der angegebene Handle nicht existiert.
                    case 1: text = boost::str( boost::format( "( %u ): Angegebener Handle ist nicht geoffnet." ) % error ); break;
                        // Wert fuer Origin ausserhalb des gueltigen Bereichs. (0, 1, 2)
                    case 2: text = boost::str( boost::format( "( %u ): Wert fuer Origin ungueltig." ) % error ); break;
                        // Sprungweite (Offset) fuehrte zu BOF (begin of file). Der Datensatzzeiger steht damit vor dem ersten Datensatz.
                    case 3: text = boost::str( boost::format( "( %u ): Sprung fuehrte zu BOF." ) % error ); break;
                        // Sprungweite (Offset) fuehrte zu EOF (end of file). Der Datensatzzeiger steht damit hinter dem letzten Datensatz.
                    case 4: text = boost::str( boost::format( "( %u ): Sprung fuehrte zu EOF." ) % error ); break;
                        // Sprung konnte aufgrund von angegebenen offset mit origin nicht durchgefuehrt werden.
                        // Z. B. Filter gesetzt und Sprung auf aktuellen der nicht dem Filterkriterium entspraeche.
                    case 5: text = boost::str( boost::format( "( %u ): Sprung konnte nicht durchgefuehrt werden." ) % error ); break;

                    default: text = boost::str( boost::format( "( %u ): Nicht eingetragener Funktionsfehler!" ) % error ); break;
                }
                break;

            case IV_FUNCTION_TABLE_SET_CURRENT_COLUMN_DATA_1:
            case IV_FUNCTION_TABLE_SET_CURRENT_COLUMN_DATA:
                switch ( error )
                {
                    // Wenn der angegebene Handle nicht existiert.
                    case 1: text = boost::str( boost::format( "( %u ): Angegebener Handle ist nicht geoffnet." ) % error ); break;
                        // Spaltenname wurde nicht gefunden.
                    case 2: text = boost::str( boost::format( "( %u ): Spaltenname unbekannt." ) % error ); break;
                        // Angegebener Wert ist zu lang
                    case 3: text = boost::str( boost::format( "( %u ): Zu setzender Wert ist zu lang." ) % error ); break;
                        // Formatfehler in zu setzendem Spaltenwert.
                    case 4: text = boost::str( boost::format( "( %u ): Formatfehler in zu setzendem Spaltenwert." ) % error ); break;
                        // Schreiben des Spaltenwerts fehlgeschlagen
                    case 5: text = boost::str( boost::format( "( %u ): Schreiben des Spaltenwerts fehlgeschalgen." ) % error ); break;

                    default: text = boost::str( boost::format( "( %u ): Nicht eingetragener Funktionsfehler!" ) % error ); break;
                }
                break;

            case IV_FUNCTION_TABLE_SET_CURRENT_ROW_DATA_1:
                switch ( error )
                {
                    // Wenn der angegebene Handle nicht existiert.
                    case 1: text = boost::str( boost::format( "( %u ): Angegebener Handle ist nicht geoffnet." ) % error ); break;
                        // Fehler beim Oeffnen der Datei.
                    case 2: text = boost::str( boost::format( "( %u ): Fehler beim Oeffnen der Datei." ) % error ); break;
                        // Datei ist Leer.
                    case 3: text = boost::str( boost::format( "( %u ): Keine Daten verfuegbar." ) % error ); break;
                        // Sprungweite (Offset) fuehrte zu EOF (end of file). Der Datensatzzeiger steht damit hinter dem letzten Datensatz.
                    case 4: text = boost::str( boost::format( "( %u ): Sprung fuehrte zu EOF." ) % error ); break;
                        // Fehler beim Lesen der Datenzeile
                    case 24: text = boost::str( boost::format( "( %u ): Fehler beim Lesen der Datenzeile." ) % error ); break;
                        // Datenzeile ist als geloescht markiert
                    case 26: text = boost::str( boost::format( "( %u ): Datenzeile ist als geloescht markiert." ) % error ); break;
                        // Allgemeiner Fehler
                    case 130:
                    case 131: text = boost::str( boost::format( "( %u ): Allgemeiner Fehler." ) % error ); break;

                    default: text = boost::str( boost::format( "( %u ): Nicht eingetragener Funktionsfehler!" ) % error ); break;
                }
                break;
                break;

            case IV_FUNCTION_TABLE_GET_CURRENT_COLUMN_DATA_1:
            case IV_FUNCTION_TABLE_GET_CURRENT_COLUMN_DATA:
            case IV_FUNCTION_TABLE_GET_CURRENT_ROW_DATA_1:
                switch ( error )
                {
                    // Wenn der angegebene Handle nicht existiert.
                    case 1: text = boost::str( boost::format( "( %u ): Angegebener Handle ist nicht geoffnet." ) % error ); break;
                        // Fehler beim Oeffnen der Datei.
                    case 2: text = boost::str( boost::format( "( %u ): Fehler beim Oeffnen der Datei." ) % error ); break;
                        // Datei ist Leer.
                    case 3: text = boost::str( boost::format( "( %u ): Keine Daten verfuegbar." ) % error ); break;
                        // Sprungweite (Offset) fuehrte zu EOF (end of file). Der Datensatzzeiger steht damit hinter dem letzten Datensatz.
                    case 4: text = boost::str( boost::format( "( %u ): Sprung fuehrte zu EOF." ) % error ); break;
                        // Fehler beim Lesen der Datenzeile
                    case 24: text = boost::str( boost::format( "( %u ): Fehler beim Lesen der Datenzeile." ) % error ); break;
                        // Datenzeile ist als geloescht markiert
                    case 26: text = boost::str( boost::format( "( %u ): Datenzeile ist als geloescht markiert." ) % error ); break;

                    default: text = boost::str( boost::format( "( %u ): Nicht eingetragener Funktionsfehler!" ) % error ); break;
                }
                break;

            case IV_FUNCTION_TABLE_GET_CURRENT_ROW_DATA:
                switch ( error )
                {
                    // Wenn der angegebene Handle nicht existiert.
                    case 1: text = boost::str( boost::format( "( %u ): Angegebener Handle ist nicht geoffnet." ) % error ); break;
                        // Fehler kein Speicher mehr.
                    case 4: text = boost::str( boost::format( "( %u ): Fehler beim reservieren von Speicher." ) % error ); break;
                        // Fehler beim Lesen der Datenzeile.
                    case 20: text = boost::str( boost::format( "( %u ): Feghler beim Lesend er Datenzeile." ) % error ); break;
                        // Sprungweite (Offset) fuehrte zu EOF (end of file). Der Datensatzzeiger steht damit hinter dem letzten Datensatz.
                    case 21: text = boost::str( boost::format( "( %u ): Sprung fuehrte zu EOF." ) % error ); break;
                        // Fehler beim Lesen der Datenzeile
                    case 22: text = boost::str( boost::format( "( %u ): Fehler beim Lesen der Datenzeile." ) % error ); break;
                        // Allgemeiner Fehler beim Lesen der Datenzeile
                    case 23: text = boost::str( boost::format( "( %u ): Allgemeiner Fehler beim Lesen der Datenzeile." ) % error ); break;
                        // Datenzeile ist als geloescht markiert
                    case 26: text = boost::str( boost::format( "( %u ): Datenzeile ist als geloescht markiert." ) % error ); break;

                    default: text = boost::str( boost::format( "( %u ): Nicht eingetragener Funktionsfehler!" ) % error ); break;
                }
                break;

            case IV_FUNCTION_TABLE_GENERATE:
            case IV_FUCNTION_TABLE_GENERATE_OK:
                switch ( error )
                {
                    case 7: text = boost::str( boost::format( "( %u ): Das Modul kann zur Zeit nicht angesprochen werden." ) % error ); break;
                    case 8: text = boost::str( boost::format( "( %u ): Fuer die erforderliche Funktion steht nicht ausreichend Speicher zur verfuegung." ) % error ); break;
                    case 129: text = boost::str( boost::format( "( %u ): Das Modul bringt einen nicht näher spezifizierten Fehler." ) % error ); break;
                    case 133: text = boost::str( boost::format( "( %u ): Das Modul enthält keine Daten zum erzeugen einer Liste." ) % error ); break;
                    default: text = boost::str( boost::format( "( %u ): Nicht eingetragener Funktionsfehler!" ) % error ); break;
                }
                break;


            case IV_FUNCTION_FINGERPRINT_TABLE_APPEND_RECORD:
            case IV_FUNCTION_FINGERPRINT_TABLE_APPEND_RECORD_OK:
            case IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD1:
            case IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD:
            case IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD_OK:
            case IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD1:
            case IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD:
            case IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD_OK:
                switch ( error )
                {
                    case 1: text = boost::str( boost::format( "( %u ): Das gewählte Templateformat ist unbekannt." ) % error ); break;
                    case 2: text = boost::str( boost::format( "( %u ): Das Modul antwortet nicht auf die Anfrage." ) % error ); break;
                    case 3: text = boost::str( boost::format( "( %u ): Das gewählte Templateformat entspricht nicht dem verwendeten." ) % error ); break;
                    case 7: text = boost::str( boost::format( "( %u ): Das Modul kann zur Zeit nicht angesprochen werden." ) % error ); break;
                    case 129: text = boost::str( boost::format( "( %u ): Das Modul bringt einen nicht näher spezifizierten Fehler." ) % error ); break;
                    case 132: text = boost::str( boost::format( "( %u ): Die Datenbank des Moduls ist voll." ) % error ); break;
                    case 133: text = boost::str( boost::format( "( %u ): Das Modul enthält keine Fingertemplates." ) % error ); break;
                    case 134: text = boost::str( boost::format( "( %u ): Das Fingertemplate existiert bereits." ) % error ); break;
                    case 135: text = boost::str( boost::format( "( %u ): Das angeforderte Fingertemplate existiert nicht." ) % error ); break;
                    default: text = boost::str( boost::format( "( %u ): Nicht eingetragener Funktionsfehler!" ) % error ); break;
                }
                break;

            case IV_FUNCTION_TABLE_GET_ROW_COUNT:
            case IV_FUNCTION_TABLE_GET_CURRENT_ROW:
                switch ( error )
                {
                    // Wenn der angegebene Handle nicht existiert.
                    case 1: text = boost::str( boost::format( "( %u ): Angegebener Handle ist nicht geoffnet." ) % error ); break;
                        // Fehler im Funktionsaufruf.
                    case 255: text = boost::str( boost::format( "( %u ): Fehler im Funktionsaufruf." ) % error ); break;

                    default: text = boost::str( boost::format( "( %u ): Nicht eingetragener Funktionsfehler!" ) % error ); break;
                }
                break;

            case IV_FUNCTION_TABLE_APPEND_ROW_DATA:
                switch ( error )
                {
                    // Wenn der angegebene Handle nicht existiert.
                    case 1: text = boost::str( boost::format( "( %u ): Angegebener Handle ist nicht geoffnet." ) % error ); break;
                        // Fehler kein Speicher verfuegbar.
                    case 4: text = boost::str( boost::format( "( %u ): Fehler kein Speicher verfuegbar." ) % error ); break;
                        // Fehler bei Funktionsausfuehrung.
                    case 3:
                    case 5:
                    case 7:
                    case 9:
                    case 10: text = boost::str( boost::format( "( %u ): Fehler bei Funktionsausfuehrung." ) % error ); break;
                        // Datenzeile ist als geloescht markiert
                    case 26: text = boost::str( boost::format( "( %u ): Datenzeile ist als geloescht markiert." ) % error ); break;

                    default: text = boost::str( boost::format( "( %u ): Nicht eingetragener Funktionsfehler!" ) % error ); break;
                }
                break;

            case IV_FUNCTION_TABLE_DELETE_CURRENT_ROW:
            case IV_FUNCTION_TABLE_DELETE_AVAILABLE_ROWS:
            case IV_FUNCTION_TABLE_DELETE_AVAILABLE_ROWS_OK:
                switch ( error )
                {
                    // Wenn der angegebene Handle nicht existiert.
                    case 1: text = boost::str( boost::format( "( %u ): Angegebener Handle ist nicht geoffnet." ) % error ); break;
                        // Sprungweite (Offset) fuehrte zu EOF (end of file). Der Datensatzzeiger steht damit hinter dem letzten Datensatz.
                    case 14: text = boost::str( boost::format( "( %u ): Sprung fuehrte zu EOF." ) % error ); break;
                        // Allgemeiner Fehler beim loeschen der Datenzeile
                    case 12:
                    case 13:
                    case 30: text = boost::str( boost::format( "( %u ): Allgemeiner Fehler beim loeschen der Datenzeile." ) % error ); break;
                        // Kein Filter aktiv.
                    case 31: text = boost::str( boost::format( "( %u ): Kein Filter aktiv." ) % error ); break;

                    default: text = boost::str( boost::format( "( %u ): Nicht eingetragener Funktionsfehler!" ) % error ); break;
                }
                break;

            case IV_FUNCTION_TABLE_SET_CURRENT_ROW_DATA:
                switch ( error )
                {
                    // Wenn der angegebene Handle nicht existiert.
                    case 1: text = boost::str( boost::format( "( %u ): Angegebener Handle ist nicht geoffnet." ) % error ); break;
                        // Fehler kein Speicher verfuegbar.
                    case 4: text = boost::str( boost::format( "( %u ): Fehler kein Speicher verfuegbar." ) % error ); break;
                        // Handlefehler
                    case 15:
                    case 16: text = boost::str( boost::format( "( %u ): Handlefehler." ) % error ); break;
                    case 17: text = boost::str( boost::format( "( %u ): Dateifehler." ) % error ); break;
                    case 18:
                    case 19: text = boost::str( boost::format( "( %u ): Lesefehler." ) % error ); break;
                        // Datenzeile ist als geloescht markiert
                    case 26: text = boost::str( boost::format( "( %u ): Datenzeile ist als geloescht markiert." ) % error ); break;

                    default: text = boost::str( boost::format( "( %u ): Nicht eingetragener Funktionsfehler!" ) % error ); break;
                }
                break;

            case IV_FUNCTION_TABLE_SET_ALL_ROWS_TO_COLUMN_DATA:
            case IV_FUNCTION_TABLE_SET_ALL_ROWS_TO_COLUMN_DATA_OK:
                switch ( error )
                {
                    // Wenn der angegebene Handle nicht existiert.
                    case 1: text = boost::str( boost::format( "( %u ): Angegebener Handle ist nicht geoffnet." ) % error ); break;
                        // Spaltenname wurde nicht gefunden.
                    case 2: text = boost::str( boost::format( "( %u ): Spaltenname unbekannt." ) % error ); break;
                        // Angegebener Wert ist zu lang
                    case 3: text = boost::str( boost::format( "( %u ): Zu setzender Wert ist zu lang." ) % error ); break;
                        // Formatfehler in zu setzendem Spaltenwert.
                    case 4: text = boost::str( boost::format( "( %u ): Formatfehler in zu setzendem Spaltenwert." ) % error ); break;
                        // Schreiben des Spaltenwerts fehlgeschlagen
                    case 5: text = boost::str( boost::format( "( %u ): Schreiben des Spaltenwerts fehlgeschalgen." ) % error ); break;

                    default: text = boost::str( boost::format( "( %u ): Nicht eingetragener Funktionsfehler!" ) % error ); break;
                }
                break;

            default: text = boost::str( boost::format( "( %u ): Nicht eingetragener Fehlerblock für Funktion!" ) % error ); break;
        }

        return;
    }

    // Ermittlung des Fehlertextes an Basisklasse abgeben.
    ModuleApplication::errorFunction( module, func, error, text );
}

//#########################################################
//####################### TABLE ###########################
//#########################################################

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceAddress Anzusprechende Geräteadresse.
 */
int32_t Datafox::Communication::ModuleTable::remoteCall( uint16_t module, uint16_t func, uint8_t *data, uint16_t &count )
{
    if ( module == IV_MODULE_TABLE )
    {
        switch ( func )
        {
        // Ermitteln der Tabellendefinition
            case IV_FUNCTION_TABLE_GET:
                return readTableStruct( deviceNumberSender(), reinterpret_cast<char *>(data), reinterpret_cast<StructTable2 &>(data[strlen( reinterpret_cast<char *>(data) ) + 1]) );

            // Ermitteln der Felddefinition
            case IV_FUNCTION_TABLE_GET_COLUMN:
                return readTableColumnStruct( deviceNumberSender(), reinterpret_cast<char *>(data), reinterpret_cast<StructTable2Field &>(data[strlen( reinterpret_cast<char *>(data) ) + 1]) );

            case IV_FUNCTION_TABLE_OPEN:
            case IV_FUNCTION_TABLE_CLOSE:
            case IV_FUNCTION_TABLE_SET_FILTER:
            case IV_FUNCTION_TABLE_SET_FILTER_OK:
            case IV_FUNCTION_TABLE_GET_FILTER:
            case IV_FUNCTION_TABLE_REMOVE_FILTER:
            case IV_FUNCTION_TABLE_SET_CURRENT_ROW:
            case IV_FUNCTION_TABLE_SET_CURRENT_COLUMN_DATA:
            case IV_FUNCTION_TABLE_GET_CURRENT_COLUMN_DATA:
            case IV_FUNCTION_TABLE_SET_CURRENT_ROW_DATA:
            case IV_FUNCTION_TABLE_GET_CURRENT_ROW_DATA:
            case IV_FUNCTION_TABLE_GENERATE:
                // Funktion nicht definiert
                return 0x0200;

            case IV_FUCNTION_TABLE_GENERATE_OK:
                return replyGenerateTable( deviceNumberSender() );

            case IV_FUNCTION_FINGERPRINT_TABLE_APPEND_RECORD:
                // Funktion nicht definiert
                return 0x0200;

            case IV_FUNCTION_FINGERPRINT_TABLE_APPEND_RECORD_OK:
                return replyAppendFingerprintRecord( deviceNumberSender() );

            case IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD:
                // Funktion nicht definiert
                return 0x0200;

            case IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD_OK:
                return replyReadFingerprintRecord( deviceNumberSender() );

            case IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD:
                // Funktion nicht definiert
                return 0x0200;

            case IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD_OK:
                return replyDeleteFingerprintRecord( deviceNumberSender() );

            case IV_FUNCTION_TABLE_GET_ROW_COUNT:
            case IV_FUNCTION_TABLE_GET_CURRENT_ROW:
                // Funktion nicht definiert
                return 0x0200;

            case IV_FUNCTION_TABLE_SET_ALL_ROWS_TO_COLUMN_DATA:
                return replyWriteAllTableRowsColumnData( deviceNumberSender() );

            case IV_FUNCTION_TABLE_DELETE_AVAILABLE_ROWS:
                return replyDeleteFiltratedTableRows( deviceNumberSender() );

            // Funktion nicht definiert
            default: return 0x0200;
        }

    }

    return ModuleApplication::remoteCall( module, func, data, count );
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceAddress Anzusprechende Geräteadresse.
 */
int32_t Datafox::Communication::ModuleTable::readTableStruct( uint8_t deviceAddress, const std::string &name, StructTable2 &table )
{
    trace( "%s(%i):INFO(3): %s [ %i, %s, STable2 ]\n", __FILE__, __LINE__, __func__, deviceAddress, name.c_str() );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        // Name als Parameter eintragen
        std::copy( name.cbegin(), name.cend(), &m_packetData[0] );
        count += static_cast<uint16_t>(name.size());
        m_packetData[count] = 0; // Nullterminierung
        ++count;

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET, result );
            return result;
        }

        // Strukturgröße prüfen
        if ( count != sizeof( StructTable2 ) )
        {
            trace( "%s(%i):ERROR(1): %s [ Die ermittelte Tabellestruktur ist ungleich der erwarteten groesse. ]\n", __FILE__, __LINE__, __func__ );
            return -IV_ERROR_PROCESSING_DLL;
        }

        // Struktur uebernehmen
        table = *reinterpret_cast<StructTable2 *>(m_packetData);
        // Kleine Prüfung auf korrekten Dateninhalt
        if ( table.number >= MAX_RECORD_DESCRIPTION 
             || !strlen( table.name ) || strlen( table.name ) > sizeof( table.name )
             || /*!table.count ||*/ table.count > MAX_FIELDS // Mit B_1015 wurden Beschreibungen ohne Felder wieder zugelassen.
             || ((table.flags & TF_SORT) && table.sort > table.count) )
        {
            trace( "%s(%i):ERROR(1): %s [ Die ermittelte Tabellenstruktur beinhaltet ungueltige Daten. ]\n", __FILE__, __LINE__, __func__ );
            return -IV_ERROR_PROCESSING_DLL;
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t data[256];
        StructTable2 t;
        count = 0;
        short err = IV_ERROR_SUCCESS;

        strcpy( reinterpret_cast<char *>(data), name.c_str() );
        memset( &t, 0, sizeof( StructTable2 ) );

        char *major;
        char *minor;

        major = strtok( reinterpret_cast<char *>(data), "." );
        minor = strtok( NULL, "." );

        if ( strcmp( major, "RECORD" ) == 0 )
        {
            switch ( atoi( minor ) )
            {
                case 0:
                    t.numberber = 0;
                    strcpy( t.name, "PZE-Meldung" );
                    t.count = 5;
                    break;

                case 1:
                    t.number = 1;
                    strcpy( t.name, "Status" );
                    t.count = 8;
                    break;

                default:
                    // Minorbestandteil undefiniert
                    err = 0x0002;
            }
        }
        else if ( strstr( reinterpret_cast<char *>(data), "LIST" ) == reinterpret_cast<char *>(data) )
        {
                // Minorbestandteil undefiniert
            err = 0x0002;
        }
        else if ( strstr( reinterpret_cast<char *>(data), "ACCESS" ) == reinterpret_cast<char *>(data) )
        {
                // Minorbestandteil undefiniert
            err = 0x0002;
        }
        else
        {
            // Majorbestandteil undefiniert.
            err = 0x0001;
        }

        if ( err == IV_ERROR_SUCCESS )
        {
            memcpy( &data[count], &t, sizeof( StructTable2 ) );
            count = sizeof( StructTable2 );
        }
        else
        {
            *reinterpret_cast<short *>(data) = err;
            count = sizeof( short );
        }
        // Antwort des Befehls senden.
        if ( (result = call( (err == IV_ERROR_SUCCESS) ? IV_CONTEXT_TRANSMIT_ANSWER : IV_CONTEXT_TRANSMIT_ANSWER_ERROR, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET, data, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceAddress Anzusprechende Geräteadresse.
 */
int32_t Datafox::Communication::ModuleTable::readTableColumnStruct( uint8_t deviceAddress, const std::string &name, StructTable2Field &field )
{
    trace( "%s(%i):INFO(3): %s [ %i, %s, StructTable2Field ]\n", __FILE__, __LINE__, __func__, deviceAddress, name.c_str() );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        // Name als Parameter eintragen
        std::copy( name.cbegin(), name.cend(), &m_packetData[0] );
        count += static_cast<uint16_t>(name.size());
        m_packetData[count] = 0; // Nullterminierung
        ++count;

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_COLUMN, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_COLUMN, result );
            return result;
        }

        // Strukturgröße prüfen
        if ( count != sizeof( StructTable2Field ) )
        {
            trace( "%s(%i):ERROR(1): %s [ Die ermittelte Feldstruktur ist ungleich der erwarteten groesse. ]\n", __FILE__, __LINE__, __func__ );
            return -IV_ERROR_PROCESSING_DLL;
        }

        // Sturktur uebernehmen
        field = *reinterpret_cast<StructTable2Field *>(m_packetData);
        // Kleine Prüfung auf korrekten Dateninhalt
        if ( field.number >= MAX_FIELDS
             || !strlen( field.name ) || strlen( field.name ) > sizeof( field.name ) 
             || strlen( field.format ) > sizeof( field.format ) )
        {
            trace( "%s(%i):ERROR(1): %s [ Die ermittelte Feldstruktur beinhaltet ungueltige Daten. ]\n", __FILE__, __LINE__, __func__ );
            return -IV_ERROR_PROCESSING_DLL;
        }

        int32_t typeOrSizeFail = 0;
        // Prüfe Feldtype und Feldgröße aufgrund von Feldtypen.
        switch ( field.type )
        {
            case TF_ULONG:
                if ( field.size != TF_ULONG_LEN )
                {
                    typeOrSizeFail = 1;
                }
                break;

            case TF_DT:
                if ( field.size != TF_DT_LEN )
                {
                    typeOrSizeFail = 2;
                }
                break;

            case TF_NUM:
                if ( field.size < TF_NUM_LEN_MIN || field.size > TF_NUM_LEN_MAX )
                {
                    typeOrSizeFail = 3;
                }
                break;

            case TF_A_NUM:
                if ( field.size < TF_A_NUM_LEN_MIN || field.size > TF_A_NUM_LEN_MAX )
                {
                    typeOrSizeFail = 4;
                }
                break;

            case TF_DATE:
                if ( field.size != TF_DATE_LEN )
                {
                    typeOrSizeFail = 5;
                }
                break;

            case TF_TIME:
                if ( field.size != TF_TIME_LEN )
                {
                    typeOrSizeFail = 6;
                }
                break;

            case TF_FINGERTEMPLATE_DIN:
                if ( field.size != TF_FINGERTEMPLATE_DIN_LEN )
                {
                    typeOrSizeFail = 7;
                }
                break;

            case TF_FINGERTEMPLATE_IDENCOM_COMPACT:
                if ( field.size != TF_FINGERTEMPLATE_IDENCOM_COMPACT_LEN )
                {
                    typeOrSizeFail = 8;
                }
                break;

            case TF_BINARY:
                if ( field.size < TF_BINARY_LEN_MIN || field.size > TF_BINARY_LEN_MAX )
                {
                    typeOrSizeFail = 9;
                }
                break;

            case TF_DATA:
                if ( field.size < TF_DATA_LEN_MIN || field.size > TF_DATA_LEN_MAX )
                {
                    typeOrSizeFail = 10;
                }
                break;

            default:
                typeOrSizeFail = 11;
        }

        if ( typeOrSizeFail )
        {
            trace( "%s(%i):ERROR(1): %s [ Die ermittelte Feldstruktur beinhaltet ungueltige Daten. Grund (%i) ]\n", __FILE__, __LINE__, __func__, typeOrSizeFail );
            return -IV_ERROR_PROCESSING_DLL;
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t data[256];
        StructTable2Field f;
        count = 0;
        short err = IV_ERROR_SUCCESS;

        strcpy( reinterpret_cast<char *>(data), name.c_str() );
        memset( &field, 0, sizeof( field ) );
        char *major;
        char *minor;
        char *subminor;

        major = strtok( reinterpret_cast<char *>(data), "." );
        minor = strtok( NULL, "." );
        subminor = strtok( NULL, "." );

        if ( strcmp( major, "RECORD" ) == 0 )
        {
            switch ( atoi( minor ) )
            {
                case 0:
                {
                    switch ( atoi( subminor ) )
                    {
                        case 0:
                            f.number = 0;
                            strcpy( f.name, "DU" );
                            f.type = TF_DT;
                            f.size = 7;
                            break;

                        case 1:
                            f.number = 1;
                            strcpy( f.name, "Kennung" );
                            f.type = TF_A_NUM;
                            f.size = 5;
                            break;

                        case 2:
                            f.number = 2;
                            strcpy( f.name, "PID" );
                            f.type = TF_A_NUM;
                            f.size = 9;
                            break;

                        case 3:
                            f.number = 3;
                            strcpy( f.name, "IP" );
                            f.type = TF_A_NUM;
                            f.size = 21;
                            break;

                        case 4:
                            f.number = 4;
                            strcpy( f.name, "Port" );
                            f.type = TF_A_NUM;
                            f.size = 9;
                            break;

                        default:
                            err = 0x0003;
                    }
                    break;
                }
                case 1:
                {
                    switch ( atoi( subminor ) )
                    {
                        case 0:
                            f.number = 0;
                            strcpy( f.name, "DU" );
                            f.type = TF_DT;
                            f.size = 7;
                            break;

                        case 1:
                            f.number = 1;
                            strcpy( f.name, "Feld1" );
                            f.type = TF_A_NUM;
                            f.size = 21;
                            break;

                        case 2:
                            f.number = 2;
                            strcpy( f.name, "Feld2" );
                            f.type = TF_A_NUM;
                            f.size = 21;
                            break;

                        case 3:
                            f.number = 3;
                            strcpy( f.name, "Feld3" );
                            f.type = TF_A_NUM;
                            f.size = 21;
                            break;

                        case 4:
                            f.number = 4;
                            strcpy( f.name, "Feld4" );
                            f.type = TF_A_NUM;
                            f.size = 21;
                            break;

                        case 5:
                            f.number = 5;
                            strcpy( f.name, "Feld5" );
                            f.type = TF_A_NUM;
                            f.size = 21;
                            break;
                        case 6:
                            f.number = 6;
                            strcpy( f.name, "Feld6" );
                            f.type = TF_A_NUM;
                            f.size = 21;
                            break;

                        case 7:
                            f.number = 7;
                            strcpy( f.name, "Feld7" );
                            f.type = TF_A_NUM;
                            f.size = 21;
                            break;

                        default:
                            err = 0x0003;
                    }
                    break;
                }
                default:
                    err = 0x0002;
            }
        }
        else if ( strcmp( major, "LIST" ) == 0 )
        {
            err = 0x0002;
        }
        else if ( strcmp( reinterpret_cast<char *>(data), "ACCESS" ) == 0 )
        {
            err = 0x0002;
        }
        else
        {
            // Majorbestandteil undefiniert.
            err = 0x0001;
        }

        if ( err == IV_ERROR_SUCCESS )
        {
            memcpy( &data[count], &f, sizeof( StructTable2Field ) );
            count = sizeof( StructTable2Field );
        }
        else
        {
            *reinterpret_cast<short *>(data) = err;
            count = sizeof( short );
        }

        // Antwort des Befehls senden.
        if ( (result = call( (err == IV_ERROR_SUCCESS) ? IV_CONTEXT_TRANSMIT_ANSWER : IV_CONTEXT_TRANSMIT_ANSWER_ERROR, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_COLUMN, data, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

/*!
    \brief Oeffnen einer Tabelle zum Bearbeiten.

    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[in] name Voll qualifizierter Name der Tabelle.
    \param[in] options Zusatzoptionen. Derzeit nicht verwendet.
    \param[out] handle Handle auf die geoeffnete Tabelle.
    \param[in, out] reserved Zeichenfolge die zur Uebertragung einer aktuell nicht verwendeten Zeichenfolge reserviert ist.
    \param[in, out] reservedSize Als Eingabe die Länge des reservierten Speichers für reserved, als Ausgabe die an reserved uebergebene Zeichenfolge.

    \retval 0 Ausfuehrung erfolgreich.
*/
int32_t Datafox::Communication::ModuleTable::openTable( uint8_t deviceAddress, const std::string &name, uint32_t options, int32_t &handle, std::string &reserved )
{
    trace( "%s(%i):INFO(3): %s [ %i, %s, %i, %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, deviceAddress, name.c_str(), options, handle, reserved.c_str() );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        // Parameter eintragen
        std::copy( name.cbegin(), name.cend(), &m_packetData[count] );
        count += static_cast<uint16_t>(name.size());
        m_packetData[count] = 0; // Nullterminierung
        count++;

        *reinterpret_cast<uint32_t *>(&m_packetData[count]) = options;
        count += sizeof( uint32_t );

        std::copy( reserved.cbegin(), reserved.cend(), &m_packetData[count] );
        count += static_cast<uint16_t>(reserved.size());
        m_packetData[count] = 0; // Nullterminierung
        count++;

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_OPEN, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_TABLE, IV_FUNCTION_TABLE_OPEN, result );
            return result;
        }

        // OK
        // Ein leerbyte fuer reservierten String in Antwort
        if ( count != (sizeof( int32_t ) + sizeof( uint8_t )) )
        {
            trace( "%s(%i):WARNING(1): %s [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, __func__, count );
        }

        // Handle uebernehmen
        handle = *reinterpret_cast<int32_t *>(m_packetData);
        // Eventuell zurueckgegebene Zeichenfolge uebernehmen.
        reserved.resize( count - sizeof( int32_t ) );
        std::copy( &m_packetData[sizeof( int32_t )], &m_packetData[count], reserved.begin() );
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_OPEN, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif

    }

    return result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceAddress Anzusprechende Geräteadresse.
 */
int32_t Datafox::Communication::ModuleTable::closeTable( uint8_t deviceAddress, int32_t handle )
{
    trace( "%s(%i):INFO(3): %s [ %i, %i ]\n", __FILE__, __LINE__, __func__, deviceAddress, handle );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );


        // Parameter eintragen
        *reinterpret_cast<int32_t *>(m_packetData) = handle;
        count = sizeof( int32_t );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_CLOSE, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_TABLE, IV_FUNCTION_TABLE_CLOSE, result );
            return result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count );
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_CLOSE, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif

    }

    return result;
}


// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceAddress Anzusprechende Geräteadresse.
 */
int32_t Datafox::Communication::ModuleTable::writeTableFilter( uint8_t deviceAddress, int32_t handle, const std::string &filter )
{
    trace( "%s(%i):INFO(3): %s [ %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, deviceAddress, filter.c_str() );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );



        // Parameter eintragen
        *reinterpret_cast<int32_t *>(m_packetData) = handle;
        count += sizeof( int32_t );
        // Filter uebernehmen
        std::copy( filter.cbegin(), filter.cend(), &m_packetData[count] );
        count += static_cast<uint16_t>(filter.size());
        m_packetData[count] = 0; // Nullterminierung
        ++count;

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_FILTER, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_FILTER, result );
            return result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count );
        }

        if ( (result = call( IV_CONTEXT_SLAVE, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_FILTER_OK, m_packetData, count, IV_PENDING_TRIALS( connection().readTimeout() ) )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceNumberReceiver(), IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_FILTER_OK, result );
            return result;
        }

        // Keine Aktive - Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s.Pending [ Aktive - Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count );
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_FILTER, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif

    }


    return result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceAddress Anzusprechende Geräteadresse.
 */
int32_t Datafox::Communication::ModuleTable::replyWriteTableFilter( uint8_t deviceAddress )
{
    trace( "%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, deviceAddress );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
#ifdef WITH_VIRTUAL_DEVICE
        trace( "%s(%i):ERROR(1): %s", __FILE__, __LINE__, __func__ );
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }
    else
    {
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_FILTER_OK, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }


    return result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceAddress Anzusprechende Geräteadresse.
 */
int32_t Datafox::Communication::ModuleTable::readTableFilter( uint8_t deviceAddress, int32_t handle, std::string &filter )
{
    trace( "%s(%i):INFO(3): %s [ %i, 0x%08X, %i ]\n", __FILE__, __LINE__, __func__, deviceAddress, filter.c_str() );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        // Parameter uebernehmen
        *reinterpret_cast<int32_t *>(m_packetData) = handle;
        count = sizeof( int32_t );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_FILTER, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_FILTER, result );
            return result;
        }

        // OK
        if ( count > 80 )
        {
            trace( "%s(%i):WARNING(1): %s [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, __func__, count );
        }

        // Wert uebernehmen
        filter = reinterpret_cast<char *>(&m_packetData[0]);
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_FILTER, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif

    }

    return result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceAddress Anzusprechende Geräteadresse.
 */
int32_t Datafox::Communication::ModuleTable::removeTableFilter( uint8_t deviceAddress, int32_t handle )
{
    trace( "%s(%i):INFO(3): %s [ %i, %i ]\n", __FILE__, __LINE__, __func__, deviceAddress, handle );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );


        // Name als Parameter eintragen
        *reinterpret_cast<int32_t *>(m_packetData) = handle;
        count = sizeof( int32_t );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_REMOVE_FILTER, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_TABLE, IV_FUNCTION_TABLE_REMOVE_FILTER, result );
            return result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count );
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_REMOVE_FILTER, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif

    }

    return result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceAddress Anzusprechende Geräteadresse.
 */
int32_t Datafox::Communication::ModuleTable::readTableRowCount( uint8_t deviceAddress, int32_t handle, uint32_t &currentCount, uint32_t &unsortedCount, uint32_t &deletedCount )
{
    trace( "%s(%i):INFO(3): %s [ %i, %i, %i, %i, %i ]\n", __FILE__, __LINE__, __func__, deviceAddress, handle, currentCount, unsortedCount, deletedCount );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        // Handle als Parameter eintragen
        *reinterpret_cast<int32_t *>(&m_packetData[count]) = handle;
        count = sizeof( int32_t );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_ROW_COUNT, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_ROW_COUNT, result );
            return result;
        }

        // OK
        if ( count > static_cast<int32_t>(sizeof( uint32_t ) + sizeof( uint32_t ) + sizeof( uint32_t )) )
        {
            trace( "%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count );
        }

        // Anzahl uebernehmen
        currentCount = *reinterpret_cast<uint32_t *>(m_packetData);
        // Ab Firmware 04.03.08 werden zusaetzliche Parameter geliefert.
        if ( count > static_cast<int32_t>(sizeof( uint32_t )) )
        {
            unsortedCount = *reinterpret_cast<uint32_t *>(&m_packetData[sizeof( uint32_t )]);
            deletedCount = *reinterpret_cast<uint32_t *>(&m_packetData[sizeof( uint32_t ) * 2]);
        }
        else
        {
            unsortedCount = 0;
            deletedCount = 0;
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_ROW_COUNT, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceAddress Anzusprechende Geräteadresse.
 */
int32_t Datafox::Communication::ModuleTable::readCurrentTableRow( uint8_t deviceAddress, int32_t handle, uint32_t &currentRow )
{
    trace( "%s(%i):INFO(3): %s [ %i, %i, %i ]\n", __FILE__, __LINE__, __func__, deviceAddress, handle, currentRow );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );


        // Name als Parameter eintragen
        *reinterpret_cast<int32_t *>(&m_packetData[count]) = handle;
        count = sizeof( int32_t );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_CURRENT_ROW, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_CURRENT_ROW, result );
            return result;
        }

        // OK
        if ( count > static_cast<int32_t>(sizeof( uint32_t )) )
        {
            trace( "%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count );
        }

        // Anzahl uebernehmen
        currentRow = *reinterpret_cast<uint32_t *>(m_packetData);
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_CURRENT_ROW, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceAddress Anzusprechende Geräteadresse.
 */
int32_t Datafox::Communication::ModuleTable::writeCurrentTableRow( uint8_t deviceAddress, int32_t handle, int32_t offset, uint16_t direction )
{
    trace( "%s(%i):INFO(3): %s [ %i, %i, %i, %i ]\n", __FILE__, __LINE__, __func__, deviceAddress, offset, direction );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );


        // Parameter eintragen
        *reinterpret_cast<int32_t *>(m_packetData) = handle;
        count = sizeof( int32_t );
        *reinterpret_cast<int32_t *>(&m_packetData[count]) = offset;
        count += sizeof( int32_t );
        *reinterpret_cast<uint16_t *>(&m_packetData[count]) = direction;
        count += sizeof( uint16_t );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_CURRENT_ROW, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_CURRENT_ROW, result );
            return result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count );
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_CURRENT_ROW, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceAddress Anzusprechende Geräteadresse.
 */
int32_t Datafox::Communication::ModuleTable::writeCurrentTableRowData( uint8_t deviceAddress, int32_t handle, const char *data, uint16_t data_size )
{
    trace( "%s(%i):INFO(3): %s [ %i, %i, 0x%08X, %i ]\n", __FILE__, __LINE__, __func__, deviceAddress, handle, data, data_size );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        // Parameter eintragen
        // Tabellen Handle
        *reinterpret_cast<int32_t *>(&m_packetData[count]) = handle;
        count += sizeof( int32_t );
        // Daten
        std::memcpy( &m_packetData[count], data, data_size );
        count += data_size;

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_CURRENT_ROW_DATA_1, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_CURRENT_ROW_DATA_1, result );
            return result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count );
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_CURRENT_ROW_DATA_1, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceAddress Anzusprechende Geräteadresse.
 */
int32_t Datafox::Communication::ModuleTable::writeCurrentTableRowDataExt( uint8_t deviceAddress, int32_t handle, uint16_t flags, char separator, const char *data, uint16_t dataSize )
{
    trace( "%s(%i):INFO(3): %s [ %i, %i, 0x%08X, %c, 0x%08X, %i ]\n", __FILE__, __LINE__, __func__, deviceAddress, handle, flags, separator, data, dataSize );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Wenn weder die flags noch der Seperator verwendet wird, auf die Vorgaengerversion der Funktion zugreifen.
    if ( flags == 0 && separator == 0 )
        return writeCurrentTableRowData( deviceAddress, handle, data, dataSize );

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );


        // Parameter eintragen
        // Tabellen Handle
        *reinterpret_cast<int32_t *>(&m_packetData[count]) = handle;
        count += sizeof( int32_t );
        // Flags
        *reinterpret_cast<uint16_t *>(&m_packetData[count]) = flags;
        count += sizeof( uint16_t );
        // Separator
        m_packetData[count] = static_cast<uint8_t>(separator);
        count++;
        // Daten
        // Zu großer Parameterwert abfangen!
        if ( dataSize > (IV_PACKET_LENGTH_COMMAND_PARAMS - count) )
        {
            return -IV_ERROR_FUNCTION_PARAMETER;
        }
        std::memcpy( &m_packetData[count], data, dataSize );
        count += dataSize;

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_CURRENT_ROW_DATA, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_CURRENT_ROW_DATA, result );
            return result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count );
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_CURRENT_ROW_DATA, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceAddress Anzusprechende Geräteadresse.
 */
int32_t Datafox::Communication::ModuleTable::writeCurrentTableColumnData( uint8_t deviceAddress, int32_t handle, const char *name, const char *data )
{
    trace( "%s(%i):INFO(3): %s [ %i, %i, 0x%08X, 0x%08X ]\n", __FILE__, __LINE__, __func__, deviceAddress, handle, name, data );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        // Name als Parameter eintragen
        *reinterpret_cast<int32_t *>(m_packetData) = handle;
        count = sizeof( int32_t );
        count += sprintf( reinterpret_cast<char *>(&m_packetData[count]), "%s", name ) + 1;
        count += sprintf( reinterpret_cast<char *>(&m_packetData[count]), "%s", data ) + 1;

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_CURRENT_COLUMN_DATA_1, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_CURRENT_COLUMN_DATA_1, result );
            return result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count );
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_CURRENT_COLUMN_DATA_1, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceAddress Anzusprechende Geräteadresse.
 */
int32_t Datafox::Communication::ModuleTable::writeCurrentTableColumnDataExt( uint8_t deviceAddress, int32_t handle, uint16_t flags, const char *name, const char *data, uint16_t dataSize )
{
    trace( "%s(%i):INFO(3): %s [ %i, %i, %i, 0x%08X, 0x%08X, %i ]\n", __FILE__, __LINE__, __func__, deviceAddress, handle, flags, name, data, dataSize );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Wenn die flags nicht verwendet werden, auf die Vorgaengerversion der Funktion zugreifen.
    if ( flags == 0 )
        return writeCurrentTableColumnData( deviceAddress, handle, name, data );

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );


        // Name als Parameter eintragen
        *reinterpret_cast<int32_t *>( m_packetData ) = handle;
        count = sizeof(int32_t);
        *reinterpret_cast<uint16_t *>( &m_packetData[count] ) = flags;
        count += sizeof(uint16_t);
        count += sprintf( reinterpret_cast<char *>( &m_packetData[count] ), "%s", name ) + 1;
        // Zu großer Parameterwert abfangen!
        if ( dataSize > (IV_PACKET_LENGTH_COMMAND_PARAMS - count) )
        {
            return -IV_ERROR_FUNCTION_PARAMETER;
        }
        memcpy(&m_packetData[count], data, dataSize);
        count += dataSize;

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_CURRENT_COLUMN_DATA, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_CURRENT_COLUMN_DATA, result );
            return result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count );
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_CURRENT_COLUMN_DATA, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceAddress Anzusprechende Geräteadresse.
 */
int32_t Datafox::Communication::ModuleTable::writeAllTableRowsColumnData( uint8_t deviceAddress, int32_t handle, uint16_t flags, const char *name, const char *data, uint16_t dataSize )
{
    trace( "%s(%i):INFO(3): %s [ %i, %i, %i, 0x%08X, 0x%08X, %i ]\n", __FILE__, __LINE__, __func__, deviceAddress, handle, flags, name, data, dataSize );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );


        // Name als Parameter eintragen
        *reinterpret_cast<int32_t *>( m_packetData ) = handle;
        count = sizeof(int32_t);
        *reinterpret_cast<uint16_t *>( &m_packetData[count] ) = flags;
        count += sizeof(uint16_t);
        count += sprintf( reinterpret_cast<char *>( &m_packetData[count] ), "%s", name ) + 1;
        // Zu großer Parameterwert abfangen!
        if ( dataSize > (IV_PACKET_LENGTH_COMMAND_PARAMS - count) )
        {
            return -IV_ERROR_FUNCTION_PARAMETER;
        }
        memcpy(&m_packetData[count], data, dataSize);
        count += dataSize;

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_ALL_ROWS_TO_COLUMN_DATA, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_ALL_ROWS_TO_COLUMN_DATA, result );
            return result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count );
        }

        // Aktive Antwort abwarten
        if ( (result = call( IV_CONTEXT_SLAVE, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_ALL_ROWS_TO_COLUMN_DATA_OK, m_packetData, count, IV_PENDING_TRIALS( connection().readTimeout() ) )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceNumberReceiver(), IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_ALL_ROWS_TO_COLUMN_DATA_OK, result );
            return result;
        }

    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_ALL_ROWS_TO_COLUMN_DATA, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceAddress Anzusprechende Geräteadresse.
 */
int32_t Datafox::Communication::ModuleTable::replyWriteAllTableRowsColumnData( uint8_t deviceAddress )
{
    trace( "%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, deviceAddress );

    int32_t result = IV_ERROR_SUCCESS;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        trace( "%s(%i):ERROR(1): %s", __FILE__, __LINE__, __func__ );
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_ALL_ROWS_TO_COLUMN_DATA_OK, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceAddress Anzusprechende Geräteadresse.
 */
int32_t Datafox::Communication::ModuleTable::readCurrentTableRowData( uint8_t deviceAddress, int32_t handle, char *data, uint16_t &dataSize )
{
    trace( "%s(%i):INFO(3): %s [ %i, %i, 0x%08X, %i ]\n", __FILE__, __LINE__, __func__, deviceAddress, handle, data, dataSize );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        // Handle als Parameter eintragen
        *reinterpret_cast<int32_t *>(&m_packetData[count]) = handle;
        count += sizeof( int32_t );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_CURRENT_ROW_DATA_1, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_CURRENT_ROW_DATA_1, result );
            return result;
        }

        // OK
        if ( count > 256 )
        {
            trace( "%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count );
        }

        if ( count > dataSize )
        {
            trace( "%s(%i):WARNING(1): %s [ Puffer für Antwortdaten zu klein gewählt. ]\n", __FILE__, __LINE__, __func__, count );
            // Wert uebernehmen
            std::copy_n( m_packetData, dataSize, data );
        }
        else
        {
            // Wert uebernehmen
            std::copy_n( reinterpret_cast<char *>(m_packetData), count, data );

            dataSize = count;
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_CURRENT_ROW_DATA_1, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceAddress Anzusprechende Geräteadresse.
 */
int32_t Datafox::Communication::ModuleTable::readCurrentTableRowDataExt( uint8_t deviceAddress, int32_t handle, uint16_t flags, char separator, char *data, uint16_t &dataSize )
{
    trace( "%s(%i):INFO(3): %s [ %i, %i, 0x%08X, %c, 0x%08X, %i ]\n", __FILE__, __LINE__, __func__, deviceAddress, handle, flags, separator, data, dataSize );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Wenn weder die flags noch der Seperator verwendet wird, auf die Vorgaengerversion der Funktion zugreifen.
    if ( flags == 0 && separator == 0 )
        return readCurrentTableRowData( deviceAddress, handle, data, dataSize );

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        // Handle als Parameter eintragen
        *reinterpret_cast<int32_t *>(&m_packetData[count]) = handle;
        count += sizeof( int32_t );
        // Flags
        *reinterpret_cast<uint16_t *>(&m_packetData[count]) = flags;
        count += sizeof( uint16_t );
        // Separator
        m_packetData[count] = static_cast<uint8_t>(separator);
        count++;

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_CURRENT_ROW_DATA, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_CURRENT_ROW_DATA, result );
            return result;
        }

        // OK
        if ( count > 256 )
        {
            trace( "%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count );
        }

        if ( count > dataSize )
        {
            trace( "%s(%i):WARNING(1): %s [ Puffer für Antwortdaten zu klein gewählt. ]\n", __FILE__, __LINE__, __func__, count );
            // Wert uebernehmen
            memcpy( data, m_packetData, dataSize );
        }
        else
        {
            // Wert uebernehmen
            memcpy( data, reinterpret_cast<char *>(m_packetData), count );
            dataSize = count;
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_CURRENT_ROW_DATA, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceAddress Anzusprechende Geräteadresse.
 */
int32_t Datafox::Communication::ModuleTable::readCurrentTableColumnData( uint8_t deviceAddress, int32_t handle, const char *name, char *data, uint16_t &dataSize )
{
    trace( "%s(%i):INFO(3): %s [ %i, %i, 0x%08X, 0x%08X, %i ]\n", __FILE__, __LINE__, __func__, deviceAddress, handle, name, data, dataSize );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        // Handle als Parameter eintragen
        *reinterpret_cast<int32_t *>(m_packetData) = handle;
        count = sizeof( int32_t );
        count += sprintf( reinterpret_cast<char *>(&m_packetData[count]), "%s", name ) + 1;

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_CURRENT_COLUMN_DATA_1, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_CURRENT_COLUMN_DATA_1, result );
            return result;
        }

        // OK
        if ( count > 440 ) // Binaerfeld mit 220 Byte als Hexstring.
        {
            trace( "%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count );
        }
        if ( count > dataSize )
        {
            trace( "%s(%i):WARNING(1): %s [ Puffer für Antwortdaten zu klein gewählt. ]\n", __FILE__, __LINE__, __func__, count );
            // Wert uebernehmen
            memcpy( data, m_packetData, dataSize );
        }
        else
        {
            // Wert uebernehmen
            memcpy( data, m_packetData, count );
            dataSize = count;
            if ( dataSize && data[dataSize] == 0 )
            {
                dataSize--; // Nullterminie abziehen.
            }
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_CURRENT_COLUMN_DATA, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceAddress Anzusprechende Geräteadresse.
 */
int32_t Datafox::Communication::ModuleTable::readCurrentTableColumnDataExt( uint8_t deviceAddress, int32_t handle, uint16_t flags, const char *name, char *data, uint16_t &dataSize )
{
    trace( "%s(%i):INFO(3): %s [ %i, %i, 0x%08X, 0x%08X, %i ]\n", __FILE__, __LINE__, __func__, deviceAddress, handle, name, data, dataSize );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Wenn die flags nicht verwendet werden, auf die Vorgaengerversion der Funktion zugreifen.
    if ( flags == 0 )
        return readCurrentTableColumnData( deviceAddress, handle, name, data, dataSize );

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        // Handle als Parameter eintragen
        *reinterpret_cast<int32_t *>(m_packetData) = handle;
        count = sizeof( int32_t );
        *reinterpret_cast<uint16_t *>(&m_packetData[count]) = flags;
        count += sizeof( uint16_t );
        count += sprintf( reinterpret_cast<char *>(&m_packetData[count]), "%s", name ) + 1;

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_CURRENT_COLUMN_DATA, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_CURRENT_COLUMN_DATA, result );
            return result;
        }

        // OK
        if ( count > 440 ) // Binaerfeld mit 220 Byte als Hexstring.
        {
            trace( "%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count );
        }
        if ( count > dataSize )
        {
            trace( "%s(%i):WARNING(1): %s [ Puffer für Antwortdaten zu klein gewählt. ]\n", __FILE__, __LINE__, __func__, count );
            // Wert uebernehmen
            memcpy( data, m_packetData, dataSize );
        }
        else
        {
            // Wert uebernehmen
            memcpy( data, m_packetData, count );
            dataSize = count;
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_CURRENT_COLUMN_DATA, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceAddress Anzusprechende Geräteadresse.
 */
int32_t Datafox::Communication::ModuleTable::appendTableRowData( uint8_t deviceAddress, int32_t handle, uint16_t flags, char separator, const char *data, uint16_t dataSize )
{
    trace( "%s(%i):INFO(3): %s [ %i, %i, 0x%08X, %c, 0x%08X, %i ]\n", __FILE__, __LINE__, __func__, deviceAddress, handle, flags, separator, data, dataSize );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        // Parameter eintragen
        // Tabellen Handle
        *reinterpret_cast<int32_t *>(&m_packetData[count]) = handle;
        count += sizeof( int32_t );
        // Flags
        *reinterpret_cast<uint16_t *>(&m_packetData[count]) = flags;
        count += sizeof( uint16_t );
        // Separator
        m_packetData[count] = static_cast<uint8_t>(separator);
        count++;
        // Daten
        // Zu großer Parameterwert abfangen!
        if ( dataSize > (IV_PACKET_LENGTH_COMMAND_PARAMS - count) )
        {
            return -IV_ERROR_FUNCTION_PARAMETER;
        }
        std::memcpy( &m_packetData[count], data, dataSize );
        count += dataSize;

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_APPEND_ROW_DATA, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_TABLE, IV_FUNCTION_TABLE_APPEND_ROW_DATA, result );
            return result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count );
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_APPEND_ROW_DATA, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceAddress Anzusprechende Geräteadresse.
 */
int32_t Datafox::Communication::ModuleTable::deleteCurrentTableRow( uint8_t deviceAddress, int32_t handle, uint16_t flags )
{
    trace( "%s(%i):INFO(3): %s [ %i, %i, , 0x%08X ]\n", __FILE__, __LINE__, __func__, deviceAddress, handle, flags );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        // Handle als Parameter eintragen
        *reinterpret_cast<int32_t *>(&m_packetData[count]) = handle;
        count = sizeof( int32_t );
        // Flags
        *reinterpret_cast<uint16_t *>(&m_packetData[count]) = flags;
        count += sizeof( uint16_t );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_DELETE_CURRENT_ROW, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_TABLE, IV_FUNCTION_TABLE_DELETE_CURRENT_ROW, result );
            return result;
        }

        // OK
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count );
        }

    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_DELETE_CURRENT_ROW, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceAddress Anzusprechende Geräteadresse.
 */
int32_t Datafox::Communication::ModuleTable::deleteFiltratedTableRows( uint8_t deviceAddress, int32_t handle, uint16_t flags )
{
    trace( "%s(%i):INFO(3): %s [ %i, %i, , 0x%08X ]\n", __FILE__, __LINE__, __func__, deviceAddress, handle, flags );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        // Handle als Parameter eintragen
        *reinterpret_cast<int32_t *>(&m_packetData[count]) = handle;
        count = sizeof( int32_t );
        // Flags
        *reinterpret_cast<uint16_t *>(&m_packetData[count]) = flags;
        count += sizeof( uint16_t );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_DELETE_AVAILABLE_ROWS, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_TABLE, IV_FUNCTION_TABLE_DELETE_AVAILABLE_ROWS, result );
            return result;
        }

        // OK
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count );
        }

        // Aktive Antwort abwarten
        if ( (result = call( IV_CONTEXT_SLAVE, IV_MODULE_TABLE, IV_FUNCTION_TABLE_DELETE_AVAILABLE_ROWS_OK, m_packetData, count, IV_PENDING_TRIALS( connection().readTimeout() ) )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceNumberReceiver(), IV_MODULE_TABLE, IV_FUNCTION_TABLE_DELETE_AVAILABLE_ROWS_OK, result );
            return result;
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_DELETE_AVAILABLE_ROWS, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceAddress Anzusprechende Geräteadresse.
 */
int32_t Datafox::Communication::ModuleTable::replyDeleteFiltratedTableRows( uint8_t deviceAddress )
{
    trace( "%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, deviceAddress );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
#ifdef WITH_VIRTUAL_DEVICE
        trace( "%s(%i):ERROR(1): %s", __FILE__, __LINE__, __func__ );
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }
    else
    {
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_DELETE_AVAILABLE_ROWS_OK, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceAddress Anzusprechende Geräteadresse.
 */
int32_t Datafox::Communication::ModuleTable::appendFingerprintRecord( uint8_t deviceAddress, uint8_t type, const uint8_t *data, uint16_t size )
{
    trace( "%s(%i):INFO(3): %s [ %i, %i, 0x%08X, %i ]\n", __FILE__, __LINE__, __func__, deviceAddress, type, data, size );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );


        // Parameter eintragen
        m_packetData[count] = type;
        count = sizeof( uint8_t );

        for ( uint16_t i = 0; i < size; ++i )
        {
            m_packetData[count + i] = data[i];
        }
        count += size;

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_APPEND_RECORD, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_APPEND_RECORD, result );
            return result;
        }

        // OK
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count );
        }

        // Aktive Antwort abwarten
        if ( (result = call( IV_CONTEXT_SLAVE, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_APPEND_RECORD_OK, m_packetData, count, IV_PENDING_FINGERPRINT_TRIALS( connection().readTimeout() ) )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceNumberReceiver(), IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_APPEND_RECORD_OK, result );
            return result;
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_APPEND_RECORD, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceAddress Anzusprechende Geräteadresse.
 */
int32_t Datafox::Communication::ModuleTable::replyAppendFingerprintRecord( uint8_t deviceAddress )
{
    trace( "%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, deviceAddress );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
#ifdef WITH_VIRTUAL_DEVICE
        trace( "%s(%i):ERROR(1): %s", __FILE__, __LINE__, __func__ );
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }
    else
    {
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_APPEND_RECORD_OK, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceAddress Anzusprechende Geräteadresse.
 */
int32_t Datafox::Communication::ModuleTable::readFingerprintRecord( uint8_t deviceAddress, uint8_t &type, uint32_t pid, uint8_t fid, uint8_t *data, uint16_t &size )
{
    trace( "%s(%i):INFO(3): %s [ %i, %i, %i, %i, 0x%08X, %i ]\n", __FILE__, __LINE__, __func__, deviceAddress, type, pid, fid, data, size );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        // Parameter eintragen
        m_packetData[count] = type;
        count = sizeof( uint8_t );

        *reinterpret_cast<uint32_t *>(&m_packetData[count]) = pid;
        count += sizeof( uint32_t );

        m_packetData[count] = fid;
        count += sizeof( uint8_t );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Wenn die Funktion unbekannt ist,
            // wird die Vorgaengerversion ohne "Aktive Antwort" geprueft.
            if ( ((result >> 8) & 0xFF) == 2 )
            {
                if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD1, m_packetData, count )) != IV_ERROR_SUCCESS )
                {
                    // Fehler ausgeben
                    traceError( deviceAddress, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD1, result );
                    return result;
                }

                // Zu kleinen Puffer abfangen!
                if ( (count - 1) > size )
                {
                    return -IV_ERROR_FUNCTION_PARAMETER;
                }

                // Templatetype uebernehmen
                type = m_packetData[0];
                // Templatedaten uebernehmen
                size = count - 1;
                for ( uint16_t i = 1; i < count; ++i )
                {
                    data[i - 1] = m_packetData[i];
                }

                // OK zurueckmelden
                return IV_ERROR_SUCCESS;
            }

            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD, result );
            return result;
        }

        // OK
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count );
        }

        // Aktive Antwort abwarten
        if ( (result = call( IV_CONTEXT_SLAVE, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD_OK, m_packetData, count, IV_PENDING_FINGERPRINT_TRIALS( connection().readTimeout() ) )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceNumberReceiver(), IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD_OK, result );
            return result;
        }

        // Zu kleinen Puffer abfangen!
        if ( (count - 1) > size )
        {
            return -IV_ERROR_FUNCTION_PARAMETER;
        }

        // Templatetype uebernehmen
        type = m_packetData[0];
        // Templatedaten uebernehmen
        size = count - 1;
        // Templatedaten muessen mindestens 15 Byte haben.
        if ( size < 15 )
        {
            return -IV_ERROR_PROCESSING;
        }

        for ( uint16_t i = 1; i < count; ++i )
        {
            data[i - 1] = m_packetData[i];
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceAddress Anzusprechende Geräteadresse.
 */
int32_t Datafox::Communication::ModuleTable::replyReadFingerprintRecord( uint8_t deviceAddress )
{
    trace( "%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, deviceAddress );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
#ifdef WITH_VIRTUAL_DEVICE
        trace( "%s(%i):ERROR(1): %s", __FILE__, __LINE__, __func__ );
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }
    else
    {
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD_OK, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceAddress Anzusprechende Geräteadresse.
 */
int32_t Datafox::Communication::ModuleTable::deleteFingerprintRecord( uint8_t deviceAddress, uint32_t pid, uint8_t fid )
{
    trace( "%s(%i):INFO(3): %s [ %i, %i, %i ]\n", __FILE__, __LINE__, __func__, deviceAddress, pid, fid );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        // Parameter eintragen
        *reinterpret_cast<uint32_t *>(&m_packetData[count]) = pid;
        count = sizeof( uint32_t );
        m_packetData[count] = fid;
        count += sizeof( uint8_t );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Wenn die Funktion unbekannt ist,
            // wird die Vorgaengerversion ohne "Aktive Antwort" geprueft.
            if ( ((result >> 8) & 0xFF) == 2 )
            {
                // Vorgaengerfunktion ausfuehren.
                if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD1, m_packetData, count )) != IV_ERROR_SUCCESS )
                {
                    // Fehler ausgeben
                    traceError( deviceAddress, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD1, result );
                    return result;
                }

                // OK
                if ( count )
                {
                    trace( "%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count );
                }

                // OK zurueckmelden
                return IV_ERROR_SUCCESS;
            }

            // Fehler ausgeben
            traceError( deviceAddress, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD, result );
            return result;
        }

        // OK
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count );
        }

        // Aktive Antwort abwarten
        if ( (result = call( IV_CONTEXT_SLAVE, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD_OK, m_packetData, count, IV_PENDING_FINGERPRINT_TRIALS( connection().readTimeout() ) )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceNumberReceiver(), IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD_OK, result );
            return result;
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceAddress Anzusprechende Geräteadresse.
 */
int32_t Datafox::Communication::ModuleTable::replyDeleteFingerprintRecord( uint8_t deviceAddress )
{
    trace( "%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, deviceAddress );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
#ifdef WITH_VIRTUAL_DEVICE
        trace( "%s(%i):ERROR(1): %s", __FILE__, __LINE__, __func__ );
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }
    else
    {
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD_OK, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceAddress Anzusprechende Geräteadresse.
 */
int32_t Datafox::Communication::ModuleTable::generateTable( uint8_t deviceAddress, const std::string &name, const std::string &reserved )
{
    trace( "%s(%i):INFO(3): %s [ %i, %s, %s ]\n", __FILE__, __LINE__, __func__, deviceAddress, name.c_str(), reserved.c_str() );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lockGuardCall guard( m_mutexCall );

        // Umgebung zuruecksetzen
        resetError();

        // Remoteadresse setzen.
        setDeviceNumberReceiver( deviceAddress );

        // Alle Zusatzparameter uebernehmen
        std::copy( name.cbegin(), name.cend(), &m_packetData[count] );
        count += static_cast<uint16_t>(name.size());
        m_packetData[count] = 0; // Nullterminierung
        count++;

        std::copy( reserved.cbegin(), reserved.cend(), &m_packetData[count] );
        count += static_cast<uint16_t>(reserved.size());
        m_packetData[count] = 0; // Nullterminierung
        count++;

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GENERATE, m_packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceNumberReceiver(), IV_MODULE_TABLE, IV_FUNCTION_TABLE_GENERATE, result );
            return result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, __func__, count );
        }
        else
        {
            count = 0;
        }

        // Aktive Antwort abwarten
        if ( (result = call( IV_CONTEXT_SLAVE, IV_MODULE_TABLE, IV_FUCNTION_TABLE_GENERATE_OK, m_packetData, count, IV_PENDING_FINGERPRINT_GENERATE_TRIALS( connection().readTimeout() ) )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceNumberReceiver(), IV_MODULE_TABLE, IV_FUCNTION_TABLE_GENERATE_OK, result );
            return result;
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GENERATE, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceAddress Anzusprechende Geräteadresse.
 */
int32_t Datafox::Communication::ModuleTable::replyGenerateTable( uint8_t deviceAddress )
{
    trace( "%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, deviceAddress );

    int32_t result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !isOwnDeviceNumber( deviceAddress ) )
    {
#ifdef WITH_VIRTUAL_DEVICE
        trace( "%s(%i):ERROR(1): %s", __FILE__, __LINE__, __func__ );
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }
    else
    {
        uint8_t tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUCNTION_TABLE_GENERATE_OK, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return result;
}
